;;PROJECT 15 (COST REDUCED SM1200) DEMODULATOR Z8 CPU PROGRAM
;;
;;VERSION 1.4   (SENT TO ZILOG FOR MASK Z8 MAY 1984)
;;
        VERS:   EQU     14H     ;;CHANGE WITH EACH NEW VERSION
        CHECKSUM:       EQU     7FFH    ;;LAST ROM BYTE IS CHECKSUM
;;*****************
;;NOTE:  THE CHECKSUM IS COMPUTED BY THE "BLOCKER" PROGRAM
;;       SUPPLIED WITH THE ADVENT UP8 EPROM PROGRAMMER.  USE
;;       METHOD #1 WHEN PROMPTED FOR CHECKSUM METHOD.
;;
;;APRIL 30 1984
;;WRITTEN BY DALE A. HEATHERINGTON
;;
;;COPYRIGHT 1984 HAYES MICROCOMPUTER PRODUCTS INC.
;;
;;
;;      REGISTER MAP
;;
        PORT0:   EQU    0       ;; I/O PORT 0
        PORT1:   EQU    1       ;; I/O PORT 1
        PORT2:   EQU    2       ;; I/O PORT 2
        PORT3:   EQU    3       ;; I/O PORT 3
        STACK:   EQU    16H     ;; TOP OF STACK 
        GLOBALS: EQU    16H     ;; GLOBAL VARIABLES (26) (EVEN START ADDRESS)
        AVGBUF: EQU     30H     ;;MOVING AVERAGE BUFFER (64 BYTES)
        INTREG:  EQU    70H     ;; INTERRUPT REGISTER SET  16 BYTES
        FLAGS:  EQU     252
;;
;;-------------------------------------------------------------------
;;GLOBAL VARIABLES
;;
        QUALACC: EQU    GLOBALS         ;;16 BIT VALUE
        QUALITY: EQU    GLOBALS+2       ;;8 BIT SIGNAL QUALITY OF LAST 256 BITS
        QUALCNT: EQU    GLOBALS+3       ;;COUNTS THE NUMBER OF AVERAGES TAKEN
        IRQ3VH: EQU     GLOBALS+4       ;;IRQ 3 HIGH VECTOR
        IRQ3VL: EQU     GLOBALS+5       ;;IRQ 3 LOW VECTOR
        RXDATA: EQU     GLOBALS+6
        RXSTAT: EQU     GLOBALS+7
        DSL:    EQU     GLOBALS+8       ;;LSB OF DESCRAMBLER
        DSM:    EQU     GLOBALS+9       ;;MIDDLE BYTE OF DESCRAMBLER
        DSH:    EQU     GLOBALS+10      ;;HIGH BYTE OF DESCRAMBLER
        DIBITS: EQU     GLOBALS+11      ;;RAW DIBITS SHIFTED IN FROM RIGHT
        P0IMAGE: EQU    GLOBALS+12      ;;USED TO SEE IF ANY LINE ON PORT 0
;;                                      ;;CHANGED SINCE LAST READ.
        TEMP1:  EQU     GLOBALS+13      ;;TEMPORARY REGISTER
        TEMP2:  EQU     GLOBALS+14
        EYEREG: EQU     GLOBALS+15      ;;HOLDING REGISTER FOR EYE OUTPUT
        IRQ4VH: EQU     GLOBALS+16      ;;IRQ4 (T0) INTERRUPT HANDLER ADDR.
        IRQ4VL: EQU     GLOBALS+17
        EQPTR:  EQU     GLOBALS+18      ;;EQUALIZER BUFFER POINTER
        DREAL:  EQU     GLOBALS+19
        STATWORD: EQU   GLOBALS+20      ;;8 BIT 1200 BAUD STATUS WORD
;;                                      ;; BITS 0-5= SIGNAL QUALITY
;;                                      ;;/ BIT   7, 6 CODE
;;                                      ;;/       1  1  CARRIER PRESENT
;;                                      ;;/       1  0  DLB REQUEST
;;                                      ;;/       0  1  DLB ACK
;;                                      ;;/       0  0  LOSS OF CARRIER
;;STATWORD MASK
        DCD:    EQU     0C0H
        DLBREQ: EQU     0BFH
        DLBACK: EQU     07FH
;;
        STATSHFT: EQU   GLOBALS+21      ;;SHIFT REGISTER FOR SERIAL CONVERSION
        SPCCTR: EQU     GLOBALS+22      ;;COUNTS SPACE BITS FOR LONG-SPACE
;;                                      ;;/  DISCONNECT  (16 BIT COUNTER)
        SWCTR:  EQU     GLOBALS+24      ;;BIT COUNTER FOR OUTPUT OF STATWORD
;;
;;---------------------------------------------------------------------
;;  PORT 0 (CONTROL INPUT FROM MASTER CPU) BITS
;;
        SQRQ:   EQU     1       ;;1=ENABLE LONG SPACE DISCONNECT        P00
        PSKFSK: EQU     2       ;;1=PSK  0=FSK                          P01
        ORGANS: EQU     4       ;;1=ORIGINATE  0=ANSWER                 P02
        BELCCI: EQU     8       ;;1=BELL 212/103  0=CCITT V.22          P03
        ALB:    EQU     10H     ;;1= ANALOG LOOP BACK                   P05
        TXFBP:  EQU     20H     ;;TX FILTER BYPASS (CALL PROGRESS)MODE  P04
        CON2:   EQU     40H     ;;CONTROL 2                             P06
        CD:     EQU     80H     ;;0=CARRIER DETECT TRUE                 P07
;;
;;      PORT 1 (CUSTOM GATE ARRAY CONTROL) BITS
;;
        FCRL:   EQU     0FH     ;;LOW 4 BITS CONTROL VCO FREQUENCY  P10-P13
        DBCLK:  EQU     10H     ;;DIBIT CLOCK OUT (600 HZ)              P14
        CPM:    EQU     20H     ;;1=CALL PROGRESS TONES, 0=DATA TONES   P15
        MODE:   EQU     40H     ;;1=2400 HZ VCO   0=1200 HZ VCO         P16
        VCO:    EQU     80H     ;;1=VCO ON    0=VCO OFF                 P17
;;
;;PORT 2 CONTROL BITS (OUTPUTS)
;;
        FILMODE: EQU    1       ;; SETS FILTER MODE 1=ANSWER, 0=ORIGINATE
        RXFBP:  EQU     2       ;; 1=BYPASS RX FILTER , 0=NORMAL

;;
;;      PORT 3 BIT INPUTS
;;
        BAUDX32: EQU    1       ;;19200 HZ INPUT FROM GATE ARRAY     P30
        RAWDAT: EQU     2       ;;GATE ARRAY PROCESSED RX DATA       P31
        LIMDAT: EQU     4       ;;HARD LIMITED RX CARRIER            P32
        SYNC:   EQU     8       ;;TELLS WHEN TO OUTPUT DATA TO "FCRL"P33
;;
;;      PORT 3 BIT OUTPUTS
;;
        STATUS: EQU     10H     ;;STATUS INFO TO MASTER CPU          P34
        CLOCK:  EQU     20H     ;;BIT CLOCK TO MASTER CPU            P35
        HZ1800: EQU     40H     ;;1800 HZ GUARD TONE OUTPUT          P36
        RDATA:  EQU     80H     ;;UNSCRAMBLED RX DATA TO MASTER CPU  P37
        
        


        ORG     0
;;
;;INTERRUPT VECTORS
;;
        DW      DM300           ;;IRQ0 P3-2 Z8 PIN 12 (+FSK CXR)
        DW      FOUT            ;;IRQ1 P3-3  (SYNC)
        DW      DM300           ;;IRQ2 P3-1 Z8 PIN 39 (-FSK CXR)
        DW      DM1200          ;;IRQ3 P3-0 Z8 PIN 5 (19200 HZ)
        DW      XIRQ4           ;;IRQ4 VECTOR FOR TIMER 0
        DW      XIRQ5           ;;IRQ5 VECTOR FOR TIMER 1
;;
        JP      START           ;;GO FOR IT!!
;;
        DB      VERS            ;;FIRMWARE VERSION NUMBER
;;
;;-------------------------------------------------------------
;;
;;ALL LOOK-UP TABLES ON PAGE ZERO TO AVOID 16 BIT ADDRESS CALCULATIONS
;;
;;TABLE USED FOR DIBIT DECODING
;;
DBDECODE:
        DB      2,0,3,1
        DB      3,2,1,0
        DB      0,1,2,3
        DB      1,3,0,2
;;
;;INITIAL VALUES FOR EQUALIZER ARRAY
;;
EQINIT: DW      0
        DW      0
        DW      0
        DW      0
        DW      0
        DW      0
        DW      0
        DW      0
;;
;;COMMAND DECODER TABLE
;;
CTABLE: DW      MAIN                    ;; 0000 INVALID
        DW      C12ANS                  ;; 0001 CCITT 1200 BAUD ANSWER
        DW      MAIN                    ;; 0010 INVALID
        DW      C12ORG                  ;; 0011 CCITT 1200 BAUD ORIGINATE
        DW      B3ANS                   ;; 0100 BELL 300 BAUD ANSWER
        DW      B12ANS                  ;; 0101 BELL 1200 BAUD ANSWER
        DW      B3ORG                   ;; 0110 BELL 300 BAUD ORIGINATE
        DW      B12ORG                  ;; 0111 BELL 1200 BAUD ORIGINATE
        DW      RINGDETECT              ;; 1000 RING DETECT MODE
        DW      LC12ANS                 ;; 1001 ALB CCITT 1200 ANS
        DW      MAIN                    ;; INVALID
        DW      LC12ORG                 ;; 1011 ALB CCITT 1200 ORG
        DW      LB3ANS                  ;; 1100 ALB BELL 300 ANS
        DW      LB12ANS                 ;; 1101 ALB BELL 1200 ANS
        DW      LB3ORG                  ;; 1110 ALB BELL 300 ORG
        DW      LB12ORG                 ;; 1111 ALB BELL 1200 ORG
;;
;;----------------------------------------------------------
XIRQ4:  JP      @IRQ4VH         ;;INDIRECT JUMP TO IRQ4 HANDLER
;;
;;START PROGRAM HERE AFTER RESET
;;
START:  DI
        CLR     IMR             ;;MASK ALL INTERRUPTS
        LD      P01M,#45H       ;;PORT1 OUTPUT, PORT0 INPUTS
        LD      PORT3,#0F0H
        LD      P3M,#1          ;;SIMPLE I/O FOR PORT 3
        LD      P2M,#0          ;;PORT 2 ALL OUTPUTS
        CLR     PORT0
        CLR     PORT2
;;
;;MEMORY SELF TEST ROUTINE
;;THE ROM CHECKSUM IS IN THE LAST BYTE OF MEMORY (7FF)
;;  / IT WAS COMPUTED AND STORED THERE BY THE "BLOCKER" PROGRAM.
;;
;;CLOCK IS PIN 10
;;RDATA IS PIN 4
;;STATUS IS PIN 29
;;
;;CLOCK GOES LOW WHEN TEST COMPLETES WITH RAM AND ROM OK, ELSE STAYS HIGH.
;;RDATA HIGH IF ROM BAD
;;STATUS HIGH IF RAM BAD
;;
SELFTEST:
        LD      PORT3,#CLOCK    ;;CLEAR RDATA AND STATUS (CLOCK HIGH)
        SRP     INTREG          ;;SET REGISTER POINTER
        LD      R0,#0FFH        ;;SET ROM POINTER TO -1
        LD      R1,#0FFH        ;; /
        CLR     R2              ;;CLEAR CHECKSUM
ROMLOOP: INCW   RR0             ;;POINT TO NEXT LOCATION
        LDC     R3,@RR0         ;;GET ROM BYTE
        ADD     R2,R3           ;;ADD TO CHECKSUM
        CP      R0,#CHECKSUM > 8
        JR      NE,ROMLOOP      ;;LOOP IF NOT DONE
        CP      R1,#CHECKSUM & 255
        JR      NE,ROMLOOP      ;; /
        CP      R2,R3           ;;TEST FOR CORRECT CHECKSUM
        JR      EQ,RAMTEST      ;;JUMP IF NO ERROR
        OR      PORT3,#RDATA    ;;SET RDATA HIGH IF ROM BAD
;;
RAMTEST:
        CLR     SPL             ;;SPL HAS REFERENCE VALUE
        LD      SPH,#4          ;;SPH POINTS TO LOCATION UNDER TEST
RTL0:   CLR     @SPH            ;;CLEAR LOCATION UNDER TEST
RTL1:   INC     @SPH            ;;INCREMENT THE LOCATION UNDER TEST
        INC     SPL             ;;INCREMENT REFERENCE VALUE
        CP      SPL,@SPH        ;;COMPARE REFERENCE WITH LOCATION UNDER TEST
        JR      NE,BADRAM       ;;EXIT IF BAD LOCATION FOUND
        CP      SPL,#0          ;;SEE IF WE HAVE DONE ALL 256 COMBOS
        JR      NZ,RTL1         ;;IF NOT THEN DO NEXT NUMBER
        INC     SPH             ;;ELSE GO ON THE NEXT RAM LOC. PAIR
        CP      SPH,#80H        ;;SEE IF WE ARE PAST TOP OF RAM
        JR      NE,RTL0         ;;IF NOT PAST TOP THEN LOOP BACK AND TEST IT
        JR      TESTDONE        ;;JUMP WHEN FINISHED
BADRAM: OR      PORT3,#STATUS   ;;SET STATUS HIGH IF RAM BAD
TESTDONE:
        TM      PORT3,#STATUS+RDATA     ;;CHECK FOR BAD ROM OR RAM
        JR      NZ,CLRMEM               ;;JUMP IF BAD LOCATION
        AND     PORT3,#"CLOCK   ;;SET CLOCK LOW ONLY WHEN RAM AND ROM ARE OK
;;
;;THE MASTER CPU IS RESPONSEABLE FOR INFORMING THE USER OF ROM OR RAM FAILURE.
;;IT TAKES 0.5 SECONDS FOR THE SELF-TEST TO BE COMPLETED
;;      
;;NOW CLEAR ALL MEMORY
;;
CLRMEM: LD      SPH,#4          ;;USE SPH FOR CLEAR OPERATION
LL000:  CLR     @SPH            ;;CLEAR ALL REGISTERS 4-7FH
        INC     SPH
        JR      PL,LL000
        SRP     INTREG          ;;SET REGISTER POINTER
        LD      SPL,#STACK      ;;SET STACK POINTER
;;
;;COMMAND DECODER.....
;;THIS ROUTINE LOOKS AT PORT 0 TO DETERMINE THE CORRECT OPERATING MODE
;;
MAIN:   LD      TEMP1,PORT0             ;;GET PORT0 IN TEMP REGISTER
        AND     TEMP1,#"(CD+SQRQ)       ;;KILL CARRIER DETECT AND SIG QUAL. BIT
        CP      P0IMAGE,TEMP1           ;;SEE IF ANY BITS ON PORT 0 CHANGED
        JR      EQ,MAIN                 ;;LOOP IF NONE CHANGED
        LD      P0IMAGE,TEMP1           ;;UPDATE RAM IMAGE OF PORT0
        TM      P0IMAGE,#CON2           ;;PORT0 DATA VALID ONLY IF CON2=1
        JR      Z,MAIN                  ;;LOOP IF NOT VALID DATA
        DI                              ;;DISABLE INTERRUPTS
        CLR     IMR                     ;;MASK ALL INTERRUPTS
        CLR     TMR                     ;;STOP ALL TIMER/COUNTERS
        LD      SPL,#STACK              ;;RESET STACK POINTER
        LD      R0,P0IMAGE
        TM      R0,#TXFBP               ;;SEE IF CALL PROGRESS MODE
        JR      NZ,CALLPROG             ;;JUMP IF IT IS.
        AND     R0,#PSKFSK+ORGANS+BELCCI+ALB  ;;KEEP OTHER MODE BITS
        LD      R2,#CTABLE > 8          ;;HIGH 8 BITS OF COMMAND TABLE BASE
        LD      R3,#CTABLE & 255        ;;LOW 8 BITS
        ADD     R3,R0                   ;;COMPUTE INDEX INTO TABLE
        LDC     R4,@RR2                 ;;GET ADDRESS FROM TABLE
        INCW    RR2                     ;;  /
        LDC     R5,@RR2                 ;; /
        JP      @RR4                    ;;INDIRECT JUMP TO ROUTINE
;;
;;
;;--------------------------------------------------------------------
;;CALL PROGRESS TONE MONITOR ROUTINE
;;
CALLPROG:       
        DI
        AND     PORT3,#"(STATUS+RDATA)  ;;STATUS AND DATA FALSE
        OR      PORT3,#CLOCK            ;;CLOCK LINE TRUE
        SRP     INTREG
        LD      IRQ4VH,#T0CP > 8        ;;SET T0 INTERRUPT DESTINATION
        LD      IRQ4VL,#T0CP & 255      ;; /
        LD      IPR,#12H
        LD      IMR,#10H                ;;ENABLE IRQ4  (T0)
        LD      PRE0,#21H               ;;PRESCALE BY 8, DIVIDE BY N
        LD      T0,#115                 ;; 1 KHZ INTERRUPT
        LD      R0,#25
        CLR     R1
        LD      TMR,#3                  ;;START T0
        EI
        JP      MAIN
;;
;;DATA LINE IS TRUE WHEN CALL PROGRESS TONE IS PRESENT
;;STATUS LINE IS TRUE WHEN DATA CARRIER IS PRESENT
;;CLOCK LINE GOES FALSE WHEN CARRIER TONE (STATUS) IS UPDATED AND
;; GOES TRUE WHEN CALL PROGREESS (RDATA) IS UPDATED
;;
T0CP:   DJNZ    R0,CPDONE
        XOR     R1,#80H
        JR      PL,LOOKCD
        LD      R0,#30                  ;;30 MS FOR DATA CARRIER DETECT
        TM      PORT0,#CD               ;;TEST CARRIER DETECT PIN
        LD      PORT2,#0                ;;SET FILTER FOR ANSWER TONE FREQ.
        LD      PORT1,#0                ;;SET FILTER CLOCK FOR DATA MODE
        JR      Z,YESCPT                ;;JUMP IF CALL PROGRESS TONE PRESENT
        AND     PORT3,#"RDATA           ;; DATA FALSE IF NO CPM TONES
        OR      PORT3,#CLOCK            ;;RAISE CLOCK LINE
        IRET
YESCPT: OR      PORT3,#RDATA            ;;DATA TRUE IF CALL PROGRESS TONES 
        OR      PORT3,#CLOCK            ;;RAISE CLOCK LINE
        IRET
;;
LOOKCD: LD      R0,#40                  ;;40 MS FOR CALL PROGRESS DETECT
        TM      PORT0,#CD               ;;TEST FOR DATA TONES
        LD      PORT2,#FILMODE          ;;SET RX FILTER FOR LOW BAND
        LD      PORT1,#CPM              ;;SET FILTER CLOCK FOR CALL PROG. MODE
        JR      Z,YESDT                 ;;JUMP IF DATA TONE PRESENT
        AND     PORT3,#"STATUS          ;;STATUS FALSE IF NO DATA TONE
        JR      CLOCKIT
YESDT:  OR      PORT3,#STATUS           ;;STATUS TRUE IF DATA TONE PRESENT
CLOCKIT: AND    PORT3,#"CLOCK           ;;INTERRUPT MASTER CPU WHEN BOTH
CPDONE: IRET                            ;; /CPM AND DATA TONES TESTED
;;
;;-------------------------------------------------------------
;;
;;INCOMING RING DETECTION ROUTINE
;;
;;WHEN RING IS DETECTED "RDATA" AND "STATUS" GO TRUE AND REMAIN TRUE
;;FOR THE DURATION OF THE RING.  "STATUS" GOES FALSE WHEN THE RING
;;STOPS BUT "RDATA" WILL REMAIN TRUE FOR 8 SECONDS AFTER THE RING STOPS.
;;THE 8 SECOND TIMER IS RESET ON EACH NEW RING.  THIS ALLOWS THE MAIN
;;CPU TO COUNT RINGS AND RESET ITS RING COUNTER THEN THE RINGS HAVE STOPPED.
;;
;;FREQUENCY MUST BE BETWEEN 15 AND 71 HZ AND LAST FOR AT LEAST
;;200 MILLISECONDS 
;;
RINGDETECT:
        DI
        LD      PORT3,#CLOCK    ;;STATUS AND RDATA FALSE
        SRP     INTREG
        LD      PORT2,#RXFBP    ;;BYPASS RECEIVE FILTER
        LD      IPR,#12H        ;;2->0->4->1->5->3
        LD      IRQ4VH,#T0RING > 8      ;;SET UP T0 INTERRUPT ADDRESS
        LD      IRQ4VL,#T0RING & 255    ;; /
        LD      IMR,#10H        ;;ENABLE IRQ4 (TIMER 0)
        LD      PRE0,#21H       ;;DIVIDE BY 8, MODULO N
        LD      T0,#115         ;;FOR 1 KHZ INTERRUPT SOURCE
        LD      TMR,#03H        ;;START TIMER 0
        CLR     R0
        CLR     R1
        CLR     R2
        CLR     R6              ;;CLEAR 8 SEC. TIMER
        CLR     R7              ;; /
        EI
        JP      MAIN
;;
;;COME HERE EVERY TIME T0 COUNTS DOWN TO ZERO (1 MS INTERVALS)
;;
T0RING:
        INC     R0              ;;BUMP 1 MS COUNTER
        JR      NC,RD1          ;;JUMP IF NO CARRY
        DEC     R0              ;;REMOVE OVERFLOW IF IT OCCURED
RD1:    CP      R0,#200         ;;TEST FOR OUT OF RANGE
        JR      ULT,RD1A        ;;INTEGRATE DOWN IF >200        
        LD      R4,#255         ;;
RD1A:   LD      R14,R15         ;;SAVE LAST SIGNAL STATUS
        LD      R15,PORT3       ;;GET LIMITER OUTPUT
        AND     R15,#LIMDAT     ;;KEEP ONLY LIMITER BIT
        TM      PORT0,#CD       ;;TEST CARRIER DETECT STATUS LINE
        JR      Z,RD1B          ;;JUMP IF TRUE
        LD      R15,#LIMDAT     ;;ELSE SET LIMITER BIT TRUE IF NO CARRIER DET.
RD1B:   XOR     R14,R15         ;;SET DIFFERENCE BITS TO 1
        TM      R14,#LIMDAT     ;;SEE IT LIMITER LINE CHANGED STATE
        JR      Z,RD2           ;;JUMP IF NO STATE CHANGE
        TM      R15,#LIMDAT     ;;SEE IF LIMITER SIGNAL IS LOW OR HIGH
        JR      NZ,RD2          ;;IGNORE LOW TO HIGH TRANSITION
        CP      R0,#67          ;;SEE IF PERIOD IS GREATER THAN 67 MS
        JR      UGT,RD3         ;;JUMP IF OUT OF RANGE
        CP      R0,#14          ;;SEE IF PERIOD LESS THAN 14 MS
        JR      ULT,RD3         ;;JUMP IF OUT OF RANGE
        CLR     R4              ;;SET TO INTEGRATE UP IF IN RANGE OF 14-67
        JR      RD2A
RD3:    LD      R4,#255         ;;SET TO INTEGRATE DOWN 
RD2A:   CLR     R0              ;;CLEAR 1 MS COUNTER
RD2:    CP      R4,#0           ;;TEST R4 TO FIND OUT WHICH WAY TO INTEGRATE
        JR      EQ,IGUP         ;;INTEGRATE UP IF ZERO
RD2B:   SUB     R1,#2           ;;ELSE INTEGRATE DOWN TWICE AS FAST
        JR      NC,RD4
        CLR     R1              ;;CLIP AT ZERO
        JR      RD4
IGUP:   INC     R1              ;;INTEGRATE UP
        JR      NZ,RD4
        LD      R1,#255         ;;CLIP AT 255
RD4:    LD      EYEREG,R1
        CALL    EYEOUT          ;;OUTPUT TO EYE GENERATOR
        CP      R1,#200         ;;CHECK THRESHOLD
        JR      ULE,RD5
        OR      PORT3,#STATUS+RDATA ;;STATUS AND RDATA=1 FOR TRUE RING DETECT
        LD      R6,#8000 > 8    ;;PRESET 8 SECOND TIMER
        LD      R7,#8000 & 255  ;; /
        IRET
RD5:    AND     PORT3,#"STATUS  ;;STATUS=0 FOR FALSE RING DETECT
        LD      R8,R7           ;;TEST R6,R7 FOR 0000H
        OR      R8,R6           ;;8 SECONDS EXPIRED? 
        JR      Z,RD6           ;;YES, CLEAR "RDATA" AND EXIT
        DECW    RR6             ;; NO, COUNT DOWN ANOTHER 1 MS
        IRET                    ;;RETURN
RD6:    AND     PORT3,#"RDATA   ;;CLEAR RDATA AFTER 8 SECONDS
        IRET
;;
;;----------------------------------------------------------------
;;OUTPUTS CONTENTS OF EYEREG TO UPPER 6 BITS OF PORT 2 WITHOUT CHANGING
;;THE 2 LOW ORDER BITS OF PORT 2.  AN EYE PATTERN GENERATOR SHOULD BE
;;CONNECTED TO THE UPPER 6 BITS TO OBSERVE THE EYE PATTERN ON A SCOPE.
;;      
EYEOUT: LD      TEMP2,PORT2
        AND     TEMP2,#FILMODE+RXFBP
        AND     EYEREG,#"(FILMODE+RXFBP)
        OR      EYEREG,TEMP2
        LD      PORT2,EYEREG
        RET
;;
;;-----------------------------------------------------------------------
;;1200 BAUD PSK INITIALIZER
;;
B12ANS:
C12ANS: LD      PORT2,#FILMODE  ;;1200 HZ RX
        LD      PORT1,#VCO      ;;1200 HZ VCO ENABLED
        JR      PSKINIT
;;
LB12ORG:
LC12ORG:LD      PORT2,#RXFBP    ;;LOOPBACK 1200HZ RX
        LD      PORT1,#VCO      ;;1200 HZ VCO ENABLED, 153.6 KHZ FILTER CLK
        JR      PSKINIT
;;
B12ORG:
C12ORG: LD      PORT2,#0        ;;FILTER IN ORIGINATE MODE (2400 HZ RX)
        LD      PORT1,#MODE+VCO ;;2400 HZ VCO ENABLED
        JR      PSKINIT
;;
LC12ANS:
LB12ANS: LD     PORT2,#FILMODE+RXFBP    ;;LOOP BACK MODE
        LD      PORT1,#MODE+VCO ;;2400 HZ VCO ENABLED, 153.6 KHZ FILTER CLK
        
;;
PSKINIT: SRP    INTREG          ;;SET REGISTER POINTER 
        LD      PORT3,#RDATA+CLOCK      ;;FORCE MARK OUTPUT     
        LD      IPR,#0BH        ;;SET INT. PRIORTY REG.
        LD      PRE0,#11H       ;;T0=MODULO N, DIVIDE BY 4 PRESCALE
        LD      T0,#64          ;;FOR 1800 HZ OUT ON P36
        LD      PRE1,#0DH       ;;INTERNAL CLK., PRESCALE=3, SINGLE PASS
        LD      T1,#255         ;;
        LD      IMR,#0AH        ;;ENABLE IRQ1,IRQ3  INTERRUPT
        LD      R0,#16
        LD      R1,P0IMAGE      ;;GET CONTROL PORT
        LD      TMR,#10H        ;;T1 GATED INPUT
        LD      R2,#AVGBUF      ;;CLEAR AVGBUF (EQUALIZER)
        LD      R4,#EQINIT > 8
        LD      R5,#EQINIT & 255
PSKI0:  LDCI    @R2,@RR4        ;;MOVE INITIAL VALUES TO EQUALIZER
        CP      R2,#AVGBUF+16
        JR      NE,PSKI0
        AND     R1,#ORGANS+BELCCI  ;;KEEP BELL/CCITT AND ORG/ANS BITS
        JR      NZ,PSKI1        ;;SEE IF CCITT ANSWER MODE, JUMP IF NOT
        LD      TMR,#53H        ;;IF ANSWER AND CCITT DO 1800 HZ OUT P36
PSKI1:  LD      STATSHFT,#0FFH
        LD      STATWORD,#0     ;;CARRIER DETECT FALSE
        LD      SWCTR,#11
        LD      R12,#325 > 8
        LD      R13,#325 & 255  ;;PRESET ALT. ONES/ZEROS COUNTER
        CLR     R8              ;;KEEP (FOUT:) ROUTINE HAPPY
        EI
        JP      MAIN            ;;JUMP TO MAIN IDLE LOOP
;;
;;---------------------------------------------------------------------
;;
;;1200 BPS PSK DEMODULATOR CODE.
;;
DM1200:         
        DJNZ    R0,CLK1         ;;JUMP IF NOT 1/2 BAUD PERIOD
        LD      R1,T1           ;;GET HARDWARE COUNTER VALUE TO R1
        OR      TMR,#0CH        ;;LOAD AND START T1
        XOR     PORT1,#10H      ;;UPDATE MUX SWITCH SIGNAL
        LD      R0,#16          ;;RESET COUNTER
        TM      PORT1,#10H      ;;TEST 600 HZ BIT
        JP      Z,BAUDEND       ;;IF ITS ZERO THEN END OF BAUD PERIOD
        LD      DREAL,R1        ;;ELSE GET REAL DATA TO "DREAL"
        CALL    EYEOUT
        NOP
        AND     PORT3,#"CLOCK   ;;SET DATA CLOCK LOW (INTERRUPT MAIN CPU)
        IRET
;;
;;COME HERE IF NOT 1/2 OR 1 BAUD PERIOD
;;
CLK1:   CP      R0,#8
        JR      NE,CP0
        OR      PORT3,#CLOCK    ;;SET DATA CLOCK HIGH
        IRET
;;
CP0:    CP      R0,#6
        JR      EQ,CP0A
        IRET
CP0A:   EI
        AND     QUALITY,#3FH    ;;LIMIT QUALITY TO 0-63 RANGE
        LD      STATWORD,#63
        SUB     STATWORD,QUALITY  ;;STATWORD = 63-QUALITY
        TM      PORT0,#CD       ;;TEST STATE OF CARRIER DETECT
        JP      NZ,CP3A         ;;JUMP IF CARRIER FALSE (FORCE MARK)
        OR      STATWORD,#DCD   ;;SET CARRIER DETECT BITS TRUE
        TM      PORT0,#SQRQ     ;;SQRQ TRUE?
        JR      Z,CP1           ;;JUMP IF LONG-SPACE DISCONN. DISABLED
        CP      SPCCTR,#8       ;;2048-128 SPACE BITS? (1.6 SECONDS)
        JR      LT,CP1          ;;JUMP IF LONG SPACE NOT DETECTED
        DECW    SPCCTR          ;;DECREMENT SPACE COUNTER
        AND     STATWORD,#"DCD  ;; CLEAR BITS 6,7
;;
;;TEST FOR UNSCRAMBLED MARKS (REMOTE DIGITAL LOOP BACK)
;;
CP1:    TM      RXDATA,#1       ;;TEST FOR MARK
        JR      NZ,CP1A         ;;JUMP IF MARK FOUND
        CLR     R6              ;;CLEAR MARK COUNTER IF SPACE DETECTED
        JR      ACKDLB          ;; / AND EXIT
CP1A:   INC     R6              ;;COUNT THE MARK
        JR      NZ,CP1B         ;;JUMP IF NO OVERFLOW
        DEC     R6              ;;IF OVERFLOW, REMOVE IT
CP1B:   CP      R6,#185         ;;SEE IF WE HAD 185 CONSECUTIVE MARKS
        JR      ULT,ACKDLB      ;;JUMP IF NOT
        AND     STATWORD,#DLBREQ ;;SET DLBREQ IF 185 MARKS DETECTED
        JR      BITSAME
;;
;;TEST FOR 325 UNSCRAMBLED ALTERNATING ONES AND ZEROS
;; FOR DIGITAL LOOP-BACK ACKNOWLEGE
;;
ACKDLB: TM      R14,#80H        ;;TEST DIFFERENCE FLAG
        JR      Z,BITSAME       ;;JUMP IF LAST TWO BITS SAME
        DECW    RR12            ;;COUNT IF DIFFERENT
        JR      NZ,CP1C         ;;JUMP IF NOT 325 BITS ONES/ZEROS
        INCW    RR12            ;;HOLD AT 1
        AND     STATWORD,#DLBACK ;;SET STATUS FOR DLB ACKNOWLEGE
        JR      CP1C
;;
BITSAME:
        LD      R12,#325 > 8    ;;RESET DIFFERENCE BIT COUNTER
        LD      R13,#325 & 255
        
;;UNSCRAMBLE DATA 
;;
CP1C:   RR      DSH             ;;PUT BIT 17 IN CARRY
        TM      DSM,#20H        ;;TEST BIT 14
        JR      Z,CP2
        CCF                     ;;XOR BIT 17 WITH BIT 14
CP2:    TM      RXDATA,#1       ;;TEST DATA BIT
        JR      Z,CP3
        CCF                     ;;XOR WITH DATA
CP3:    RRC     RXDATA          ;;SHIFT CARRY INTO MSB OF RXDATA
        ADC     DSL,DSL         ;;SHIFT DESCRAMBLER REGISTER
        ADC     DSM,DSM
        ADC     DSH,DSH
        LD      R14,PORT3       ;;GET LAST DATA BIT TO BIT 7, R12
        XOR     R14,RXDATA      ;;XOR WITH NEW DATA BIT FOR DIFFERENCE
        TM      RXDATA,#80H     ;;SEE IF DATA BIT IS ONE OR ZERO
        JR      Z,CP4
CP3A:   OR      PORT3,#RDATA    ;;OUTPUT DATA BIT HIGH
        CLR     SPCCTR          ;;CLEAR SPACE COUNTER
        LD      SPCCTR+1,#128
        JR      CP5
CP4:    AND     PORT3,#"RDATA   ;;OUTPUT DATA BIT LOW
        INCW    SPCCTR          ;;COUNT THE SPACE
;;
;;NOW OUTPUT THE STATUS BYTE SERIALLY
;;
CP5:    INC     SWCTR           ;;BUMP STATUS COUNTER
        CP      SWCTR,#1        ;;TEST FOR START BIT TIME
        JR      NE,CP6          ;;JUMP IF NOT 1 
        AND     PORT3,#"STATUS  ;;ELSE SEND A START BIT
        JR      CPX             ;;/ THEN EXIT
CP6:    CP      SWCTR,#20       ;;JUMP IF NOT 20 (END OF CYCLE)
        JR      NE,CP7
        CLR     SWCTR           ;;CLEAR COUNTER
        LD      STATSHFT,STATWORD  ;;GET NEW STATUS DATA
        JR      CPX                     ;;EXIT
CP7:    TM      STATSHFT,#1     ;;TEST LSB OF STATUS DATA
        JR      Z,CP8
        OR      PORT3,#STATUS   ;;SET STATUS BIT TRUE
        JR      CP9
CP8:    AND     PORT3,#"STATUS  ;;SET STATUS BIT FALSE
CP9:    SCF
        RRC     STATSHFT        ;;SHIFT STATUS DATA RIGHT WITH ONES FILL
CPX:    LD      R10,R0          ;;DEBUGGING AID
        IRET
;;
DECPTR: INC     EQPTR
        SUB     @EQPTR,#8
        DEC     EQPTR
        SBC     @EQPTR,#0
        CP      @EQPTR,#-10H    ;;CHECK BOUNDS
        JR      LT,INCPTR       ;;IF OUT THEN RESTORE VALUE
        RET
;;
INCPTR: INC     EQPTR
        ADD     @EQPTR,#8
        DEC     EQPTR
        ADC     @EQPTR,#0
        CP      @EQPTR,#10H     ;;CHECK BOUNDS
        JR      GT,DECPTR       ;;IF OUT THEN RESTORE VALUE
        RET
;;
;;THIS ROUTINE IS ENTERED AT THE BOUNDRY BETWEEN 2 BAUD TIMES.
;;IT COMPUTES THE PHASE CHANGE SINCE THE LAST BAUD AND CONVERTS IT TO
;;A PAIR OF BITS IN POSITIONS 0,1 OF "RXDATA".
;;IT ALSO COMPUTES THE PHASE ERROR OF THE VCO AND APPLIES A 
;;CORRECTION TO KEEP IT LOCKED TO THE INCOMING CARRIER.
;;
BAUDEND:
        LD      R3,R1           ;;SAVE IMAG. DATA TO  R3
        AND     PORT3,#"CLOCK   ;;SET DATA CLOCK LOW (INTERRUPT MAIN CPU)
        EI
        LD      R2,DREAL        ;;GET REAL DATA TO R2
        SUB     R3,#128         ;;NORMALIZE FOR +/- AROUND ZERO
        SUB     R2,#128
        LD      EQPTR,DIBITS    ;;GET OLD DIBITS FROM LAST BAUD
        ADD     EQPTR,EQPTR
        ADD     EQPTR,EQPTR     ;;MPY BY 4
        AND     EQPTR,#0CH      ;;MASK OUT UNWANTED BITS
        ADD     EQPTR,#AVGBUF   ;;ADD BASE ADDRESS TO FORM POINTER
        SUB     R2,@EQPTR       ;;SUBTRACT EQUALIZER OFFSET FROM REAL PART
        TM      R2,#40H         ;;TEST SIGN OF ERROR
        JR      Z,EQZ0
        CALL    INCPTR          ;;ADJUST EQUALIZER REAL PART
        JR      EQZ1
EQZ0:   CALL    DECPTR          ;; //
EQZ1:   
        ADD     EQPTR,#2        ;;POINT TO IMAG. EQUALIZER LOCATION
        SUB     R3,@EQPTR       ;;SUBTRACT EQUALIZER OFFSET FROM IMAG. PART
        TM      R3,#40H         ;;TEST SIGN OF ERROR
        JR      Z,EQZ2
        CALL    INCPTR          ;;ADJUST EQUALIZER IMAG. PART
        JR      EQZ3
EQZ2:   CALL    DECPTR          ;; //
EQZ3:   
        TM      PORT0,#SQRQ
        JR      Z,EQZ4
        LD      EYEREG,R3
        ADD     EYEREG,#128
        CALL    EYEOUT
        LD      EYEREG,R2
        ADD     EYEREG,#128
EQZ4:   LD      R4,R3           
        RL      R4              ;;SHIFT DIBIT INTO CARRY
        RLC     DIBITS          ;;SHIFT CARRY INTO LSB OF DIBITS
        LD      R4,R2           ;;GET OTHER DIBIT INTO R4
        RL      R4              ;;SHIFT DIBIT INTO CARRY
        RLC     DIBITS          ;;SHIFT CARRY INTO DIBITS LSB
        AND     DIBITS,#0FH     ;;KEEP LAST 2 DIBIT PAIRS
        LD      R4,#DBDECODE > 8        ;;GET BASE ADDRESS OF DECODE TABLE
        LD      R5,#DBDECODE & 255      ;; /
        ADD     R5,DIBITS       ;;INDEX INTO DECODER TABLE
        LDC     R4,@RR4         ;;GET DATA BIT PAIR FROM TABLE
        LD      RXDATA,R4       ;;PUT IT IN RXDATA LOC.
;;
;;COMPUTE VCO PHASE ERROR  (DIGITAL COSTAS LOOP)
;;
        LD      R4,R2
        TM      R3,#0FFH        ;;TEST SIGN OF R3
        JR      PL,Z1
        COM     R2
        INC     R2
Z1:     TM      R4,#0FFH
        JR      PL,Z2
        COM     R3
        INC     R3
Z2:     LD      R4,R2
;;
        SUB     R4,R3
;;      
Z2A:    TM      R4,#0FFH        ;;TEST SIGN OF ERROR
        PUSH    FLAGS           ;;SAVE SIGN OF ERROR IN FLAGS
        LD      R2,R4           ;;SAVE ERROR FOR QUALITY CALC.
        JR      PL,Z3           ;;TAKE ABSOLUTE VALUE OF ERROR
        COM     R2
        INC     R2              ;;R2= ABSOLUTE VALUE OF ERROR
Z3:     INC     QUALCNT         ;;ADVANCE AVERAGE COUNTER
        JR      NZ,Z4           ;;JUMP IF NOT 256 AVERAGES
        CLR     QUALACC
        LD      QUALITY,QUALACC+1       ;;UPDATE QUALITY REGISTER
        CLR     QUALACC+1       
Z4:     ADD     QUALACC,R2      ;;DO AVERAGE
        ADC     QUALACC+1,#0
        LD      R5,R15
        POP     FLAGS           ;;RECOVER SIGN OF ERROR FROM STACK
        JR      PL,PLS          ;;TEST SIGN OF ERROR
        ADD     R15,#2          ;;UPDATE INTEGRATOR REGISTER UP IF MINUS
        ADD     R5,#18H         ;;ADD OFFSET TO INTEGRATOR IF ERROR 
PLS:    SUB     R15,#1          ;;UPDATE INTEGRATOR DOWN IF PLUS
        CP      R15,#67H        ;;TEST FOR OUT OF RANGE INTEGRATOR
        JR      ULT,CLK2        ;;JUMP IF NOT OUT OF RANGE
        LD      R15,#3CH        ;;RESET INTEGRATOR IF OUT OF RANGE
CLK2:   RL      R5
        SWAP    R5              ;;GET BITS IN RIGHT MOST POSITION
        AND     R5,#0FH         ;;KILL ALL UNWANTED BITS
        LD      R8,R5           ;;FREQUENCY DATA TO R8
        LD      R11,R0          ;;DEBUG AID
        IRET
;;
        DB      'COPYRIGHT 1984 HAYES MICROCOMPUTER PRODUCTS, INC'
;;
FOUT:   LD      R9,PORT1
        AND     R9,#0F0H
        CP      R8,#0FH         ;;TEST FOR DISALLOWED VALUE
        JR      NE,FOK
        DEC     R8              ;;MOVE TO ALLOWED RANGE IF IT WAS OUT
FOK:    OR      R9,R8
        LD      PORT1,R9
        IRET
;;
;;
;;----------------------------------------------------------------
;;LOW SPEED FSK DEMODULATOR
;;
;;RR8 IS THE 1.6 SECOND LONG-SPACE DISCONNECT TIMER
;;
LB3ORG: LD      PORT2,#RXFBP    ;;FILTER ORIGINATE AND BYPASS
        JR      B3ANSX
;;
B3ANS:  LD      PORT2,#FILMODE  ;;FILTER ANSWER MODE
B3ANSX: LD      R6,#25H         ;;SAVE COPY OF FUTURE IMR IN R6
        LD      T0,#0EBH        ;;ZERO CROSSING TIMER STARTS AT EB
        JR      FSKINIT
;;
LB3ANS: LD      PORT2,#FILMODE+RXFBP  ;;ANSWER FILTER AND BYPASS
        JR      B3ORGX
;;
B3ORG:  LD      PORT2,#0        ;;ORIGINATE MODE FILTER
B3ORGX: LD      R6,#21H         ;;
        LD      T0,#0FFH        ;;ZERO CROSSING TIMER STARTS AT FF
FSKINIT:DI 
        SRP     INTREG
        LD      PORT3,#RDATA+CLOCK      ;;FORCE MARK CONDITION
        LD      IPR,#1CH        ;;0->2->5->3->1->4= PRIORITY
        LD      PRE0,#8         ;;DIVIDE BY 2, SINGLE PASS
        LD      PRE1,#7         ;;INTERNAL, DIV. BY 1, MODULO N
        LD      T1,#128         ;;FOR 7200 HZ INTERRUPT
        LD      PORT1,#0        ;;FSK MODE TO GATE ARRAY
        CLR     R14             ;;CLEAR MOVING AVERAGE ACCUMULATOR
        CLR     R15             ;; /
        LD      R0,#16          ;;LOAD LOOP COUNTER WITH 16
        LD      R2,#AVGBUF+16   ;;SET POINTER TO END OF ARRAY
CLRBUF: DEC     R2              ;;DECREMENT ARRAY POINTER
        CLR     @R2             ;;CLEAR MOVING AVERAGE ARRAY
        DJNZ    R0,CLRBUF       ;;DECREMENT LOOP COUNTER AND JUMP BACK
        LD      IMR,R6          ;;SET INTERRUPT MASK
        LD      TMR,#0FH        ;;START TIMER0 AND TIMER1
        CLR     IRQ             ;;CLEAR ANY PENDING INTERRUPTS
        LD      R8,#11520 > 8   ;;PRESET 1.6 SEC. LONG SPACE TIMER
        LD      R9,#11520 & 255
        CLR     R10             ;;CLEAR LONG SPACE DETECTED FLAG
        EI
        JP      MAIN
;;
DM300:  
        LD      R0,T0           ;;GET ZERO CROSSING TIME FROM T0
        OR      TMR,#3          ;;LOAD AND RESTART T0
        CP      R0,#8           ;;COMPARE WITH LOWER LIMIT
        JR      UGE,DM1         ;;JUMP IF OK
        LD      R0,#8           ;;ELSE SET TO LOWER LIMIT (8)
DM1:    CP      R0,#63          ;;COMPARE WITH UPPER LIMIT
        JR      ULT,DM2         ;;JUMP IF OK
        LD      R0,#63          ;;ELSE SET TO UPPER LIMIT
DM2:    LD      R5,R4           ;;AVERAGE WITH THE LAST ZERO CROSSING TIME
        LD      R4,R0           ;;/
        ADD     R0,R5           ;;/
        RRC     R0              ;;/
        AND     IRQ,#"5
DM3:    IRET
;;
;;16 POINT MOVING AVERAGE CALCULATION ROUTINE.
;;TAKES AN AVERAGE OF THE LAST 16 ZERO CROSSING TIME READINGS.
;;
XIRQ5:
        LD      R1,R0
        PUSH    IMR
        AND     IMR,#"20H       ;;DISABLE IRQ5
        EI
        ADD     R15,R1
        ADC     R14,#0
        SUB     R15,@R2
        SBC     R14,#0
        LD      @R2,R1
        INC     R2
        CP      R2,#AVGBUF+16
        JR      NE,XIR1
        LD      R2,#AVGBUF
XIR1:   LD      R12,R14
        LD      R13,R15
        SRA     R12
        RRC     R13
        SRA     R12
        RRC     R13
        LD      EYEREG,R13              ;;OUTPUT EYE PATTERN
        CALL    EYEOUT
        TM      PORT0,#CD               ;;TEST FOR CARRIER
        JR      Z,XIR1A                 ;;JUMP IF CARRIER PRESENT
LSDR:   AND     PORT3,#"STATUS          ;;CLEAR STATUS IF NO CARRIER
        JR      XIR2A                   ;;THEN FORCE MARK ON DATA OUTPUT
XIR1A:  RR      R10                     ;;TEST LONG-SPACE DISCONNECT FLAG
        JR      NC,XIR1B
        AND     PORT3,#"STATUS          ;;DROP STATUS LINE IF LONG SPACE
        JR      XIR1C   
XIR1B:  OR      PORT3,#STATUS           ;;SET STATUS LINE TRUE IF CARRIER 
XIR1C:  LD      R12,#0A3H               ;;SLICING LEVEL BASE VALUE
        TM      PORT3,#RDATA
        JR      Z,XIR2
        SUB     R12,#8                  ;;
XIR2:   CP      R13,R12                 ;;DETERMINE 1 OR 0
        JR      ULT,XIR3
XIR2A:  OR      PORT3,#RDATA            ;;OUTPUT 1
        LD      R8,#11520 > 8
        LD      R9,#11520 & 255         ;;PRESET SPACE COUNTER
        CLR     R10                     ;;CLEAR LONG SPACE FLAG
        JR      XIR4
XIR3:   AND     PORT3,#"RDATA           ;;OUTPUT 0
        DECW    RR8                     ;;COUNT SPACE FOR LONG-SPACE DISCONNECT
        JR      NZ,XIR4
        INCW    RR8
        TM      PORT0,#SQRQ             ;;LONG SPACE DISC. ENABLED?
        JR      Z,XIR4                  ;;JUMP IF NOT
        LD      R10,#0FFH               ;;ELSE SET LONG-SPACE FLAG
XIR4:   DI                              
        POP     IMR                     ;;ENABLE IRQ5 AGAIN
ENDROM: IRET                            ;;LAST BYTE IN PROGRAM ROM
;;
;;----------------------------------------------------------------
;;
        END

