;;PROJECT 15 (COST REDUCED SM1200) DEMODULATOR Z8 CPU PROGRAM
;;   FILE:= MODEM130.ASM            CREATED MARCH 1984
;;------------------------------------------------------------------------
;;  Low cost Smartmodem 1200 firmware
;;  SOURCE VERSION 3.3  
;;  LAST REV. 8-28-84
;;
        VERS:   EQU     127             ;;CHANGE THIS WITH EACH ROM VERSION
;;
;;  WRITTEN BY Dale Heatherington
;;
;;  Copyright 1984 by Hayes Microcomputer Products, Inc.
;;-------------------------------------------------------------------------
;; THIS IS FOR A 7.3728 MHZ CRYSTAL
;; CYCLE TIME IS 271.27 NS.
;;---------------------------------------------------
;;DATA ROM EQUATES
        TONETBL: EQU    1000H   ;;DTMF DIGIT TO FREQ. POINTER TABLE
        FRQTBL: EQU     1020H   ;;DTMF FREQUENCY TABLE
        FORMAT: EQU     1030H   ;;PARITY & WORD LENGTH TABLE
        P10TAB: EQU     1040H   ;;POWERS OF 10 TABLE
        INITDATA:EQU    1042H   ;;INITIAL S REGISTER DATA
        SINE256: EQU    1100H   ;;256 BYTE SINE TABLE
        SINE2DB: EQU    1200H   ;;2 DB LOWER LEVEL SINE TABLE
        OKMSG:  EQU     1300H   ;;RESULT CODE ASCII STRINGS
;;
;;
;;
;;      REGISTER MAP
;;
        PORT0:   EQU    0       ;; I/O PORT 0
        PORT1:   EQU    1       ;; I/O PORT 1
        PORT2:   EQU    2       ;; I/O PORT 2
        PORT3:   EQU    3       ;; I/O PORT 3
        SPARE:  EQU     4       ;; 6 MORE REGISTERS
        BUFFER:  EQU    10      ;; 40 CHARACTER INPUT BUFFER
        STACK:   EQU    40H     ;; TOP OF 20 BYTE STACK
        GLOBALS: EQU    40H     ;; GLOBAL VARIABLES 32 BYTES
        PRIMREG: EQU    60H     ;; PRIMARY REGISTER SET    16 BYTES
        INTREG:  EQU    70H     ;; INTERRUPT REGISTER SET  16 BYTES
;;
;;-------------------------------------------------------------------
;;      FREQUENCY LIST FOR 7.3728 MHZ CPU CRYSTAL
;;      WHICH IS DIVIDED DOWN TO 921.6 KHZ INTERNALLY
;;      IN THE Z8.  THE 921.6 KHZ IS DIVIDED BY 96 BY
;;      A Z8 INTERNAL TIMER-COUNTER TO YIELD 9600 HZ
;;      MAIN INTERRUPT RATE.  DIVISION BY 8 GIVES 1200 BAUD
;;      FOR THE LINE RATE.
;;      F * .853333 = THE FOLLOWING CONSTANTS.
;;
        MC9600: EQU     96              ;;9600 HZ
        MC7200: EQU     128             ;;7200 HZ
;;
        N:      EQU     8
;;FOR 9600 HZ CLOCK
        F697:   EQU     595*N
        F770:   EQU     657*N
        F852:   EQU     727*N
        F941:   EQU     803*N
        F1209:  EQU     1032*N
        F1336:  EQU     1140*N
        F1477:  EQU     1260*N
        F1633:  EQU     1393*N
        F2100:  EQU     1792*N
        F1200:  EQU     1024*N
        F2400:  EQU     2048*N
;;
;;THESE ARE FOR 7200 HZ CLOCK AND ARE ONLY USED FOR 300 BAUD FSK
;; F* 1.1379 = CONSTANTS
;;
        F1070:  EQU     1217*N
        F1270:  EQU     1445*N
        F2025:  EQU     2304*N
        F2225:  EQU     2532*N
;;
;;--------------------------------------------------------

;;PORT 2 BIT DEFINITIONS
;;    INPUTS
;;
        RDATA:  EQU     1               ;;P20 RX DATA FROM DEMOD PROCESSOR
        MUXOUT: EQU     2               ;;P21 DATA FROM OPTION SWITCH MUX
;;OUTPUTS
        DACMASK: EQU    0FCH            ;;UPPER SIX BITS ARE DAC OUTPUTS
;;
        
        DAC:    EQU     2               ;;DAC IS HOOKED TO PORT 2
;;
;;PORT 3 BIT DEFINITIONS
;;
        TXD:    EQU     1               ;;P30 TX DATA IN FROM TERMINAL
        CLOCK:  EQU     2               ;;P31 CLOCK FROM DEMOD PROCESSOR
        STATUS: EQU     4               ;;P32 STATUS FROM DEMOD PROCESSOR
        DTR:    EQU     8               ;;P33 DTR FROM RS232 INPUT
;;
;;THESE ARE THE "S" REGISTERS
;;
SR:     EQU     GLOBALS
RINGN:  EQU     SR              ;;S0  RING TO ANSWER ON
RINGS:  EQU     SR+1            ;;S1  RINGS WHICH HAVE OCCURED
ESCODE: EQU     SR+2            ;;S2  ESCAPE CHARACTER
CRC:    EQU     SR+3            ;;S3  CARRAGE RETURN CHAR.
LFC:    EQU     SR+4            ;;S4  LINE FEED CHAR.
BSC:    EQU     SR+5            ;;S5  BACK SPACE CHAR.
DTW:    EQU     SR+6            ;;S6  DIAL TONE WAIT TIME
CDW:    EQU     SR+7            ;;S7  CARRIER DETECT WAIT TIME
PT:     EQU     SR+8            ;;S8  PAUSE TIME FOR COMMA
CDELAY: EQU     SR+9            ;;S9  CARRIER DETECT RESPONSE TIME
LCDT:   EQU     SR+10           ;;S10 LOST CARRIER TO DISCONNECT TIME
TONLEN: EQU     SR+11           ;;S11 DTMF TONE DURATION AND SPACING
EGT:    EQU     SR+12           ;;S12 ESCAPE CODE GUARD TIME
USTAT:  EQU     SR+13           ;;S13 UART STATUS REGISTER
OFLAGS: EQU     SR+14           ;;S14 OPTION FLAGS
PFLAGS: EQU     SR+15           ;;S15 PROGRAM STATUS FLAGS
LPBACK: EQU     SR+16           ;;S16 SELF TEST CONTROL BYTE...
;;                              ;;    BIT 0=ANALOG LOOP BACK
;;                              ;;    BIT 1=LONG TOUCH TONE TEST MODE
;;                              ;;    BIT 2=DIGITAL LOOP BACK
;;
S16ALB: EQU     1               ;;S16 ANALOG LOOP BACK MASK
S16TTT: EQU     2               ;;S16 TOUCH TONE TEST  MASK
S16DLB: EQU     4               ;;S16 DIGITAL LOOP BACK MASK
S16RDL: EQU     8               ;;S16 REMOTE DIGITAL LOOP BACK MASK
;;
;; END OF PUBLISHED "S" REGISTERS
;;
;; BIT MAP FOR USTAT
;;
RRF:    EQU     1               ;; 1=RECEIVE REGISTER FULL
;;SPARE
PE:     EQU     4               ;;PARITY ENABLE
EPE:    EQU     8               ;;EVEN PARITY ENABLE
DBITS:  EQU     10H             ;; 1=8 DATA BITS  0=7 DATA BITS
IBF:    EQU     20H             ;;INPUT BUFFER FULL FLAG
OVL:    EQU     40H             ;;INPUT BUFFER OVERFLOW FLAG
DB8:    EQU     80H             ;;STUCK PARITY STATE OF DATA BIT 8
;;
;;-------------------------------------------------------
;; EXTENDED I/O EQUATES
;;
;;INPUT FROM CD4051 8:1 MUX (MOSTLY CONNECTED TO CONFIGRATION DIP SWITCH)
;; LOAD PORT 0 WITH THE FOLLOWING CONSTANTS THEN READ "PORT2,#MUXOUT"
;;
        RCD:    EQU     0       ;;0=RAW CARRIER DETECT
        SW5:    EQU     10H     ;;1=AUTO ANSWER
        SW4:    EQU     20H     ;;1=LOCAL ECHO
        SW3:    EQU     30H     ;;1=QUIET
        SW2:    EQU     40H     ;;1=VERBOSE
        SW8:    EQU     50H     ;;1=DUMB MODE
        JP1:    EQU     60H     ;;1=61/39 MAKE BREAK RATIO, 0=67/33
        SW9:    EQU     70H     ;;1=BELL 103/212A,  0=CCITT V.22
;;
;; BIT MAP FOR OFLAGS
;;
VBON:   EQU     8               ;;SW2 VERBOSE MODE ON
QUIET:  EQU     4               ;;SW3 TRUE = RESULT CODES OFF
LEON:   EQU     2               ;;SW4 LOCAL ECHO ON
AAON:   EQU     1               ;;SW5 AUTO ANSWER ON
STUPID: EQU     10H             ;;SW8 1=DONT EXECUTE ANY COMMANDS
MB6139: EQU     20H             ;;JP1 1=61/39% MAKE BREAK  0=67/33%
BELLOPT:EQU     40H             ;;SW9 1=BELL 0=CCITT  (SWITCH 9)
LSD:    EQU     80H             ;;1= LONG SPACE DISCONNECT ENABLED
;;
;; BITMAP FOR PFLAGS
;;
IBAUD0: EQU     1               ;;INPUT BAUD RATE SELECT BIT 0
IBAUD1: EQU     2               ;;INPUT BAUD RATE SELECT BIT 1
MODE:   EQU     4               ;;1=ORIG   0=ANSW
DPLX:   EQU     8               ;;1=FULL DUPLEX   0=HALF DUPLEX
OBAUD0: EQU     10H             ;;OUTPUT BAUDRATE SELECT BIT 0
OBAUD1: EQU     20H             ;;OUTPUT BAUDRATE SELECT BIT 1
CONEST: EQU     40H             ;;1=CONNECTION ESTABLISHED FLAG
ONLINE: EQU     80H             ;;1=ON-LINE   0=LOCAL COMMAND MODE
;;
;;.....BIT MAP OF PFLAGS2
TDIAL:  EQU     1               ;;1=TONE DIALING 0=PULSE
TCA:    EQU     2               ;;TRANSMITTER CHARACTER AVALABLE FLAG
CDLED:  EQU     4               ;;PLACE TO SAVE STATE OF CD LED
WDT:    EQU     8               ;;WAIT FOR DIAL TONE ENABLE FLAG
BYCXR:  EQU     10H             ;;ENABLE BUSY DETECT DURING CARRIER DETECT
EXR:    EQU     20H             ;;EXTENDED RESULT CODES ENABLE
XVOL0:  EQU     40H             ;;SPEAKER VOLUME CONTROL LSB
XVOL1:  EQU     80H             ;;SPEAKER VOLUME CONTROL MSB
;;
;;.....BIT MAP OF PFLAGS3 (WARNING: POSITION OF BITS CRITICAL!!)
;;
SUSM:   EQU     3               ;;LOW 2 BITS=1 = SEND UNSCRAMBLED MARK
ONEZERO: EQU    8               ;;1=SEND ALTERNATING ONES AND ZEROS
SBREAK: EQU     0CH             ;;00001100 = SEND BREAK
MOD$DIS: EQU    10H             ;;1=DISABLE HIGH SPEED MODULATON
TXON:   EQU     20H             ;;1=MODEM CARRIER ENABLED
SPON:   EQU     40H             ;;SPEAKER ON UNTIL CARRIER DETECTED
SPON2:  EQU     80H             ;;SPEAKER ON ALWAYS
;;
;;GLOBAL REGISTER EQUATES
;;
;;
IRQ5VH: EQU     SPARE+0         ;;DESTINATION FOR TIMER 1 INTERRUPT
IRQ5VL: EQU     SPARE+1
PFLAGS3: EQU    SPARE+2         ;;THIRD SET OF PROGRAM STATUS FLAGS
PFLAGS2: EQU    SPARE+3         ;;SECOND SET OF PROGRAM STATUS FLAGS
COUNTER: EQU    SPARE+5
;;
;;
GL:     EQU     51H
IMAGE35:EQU     GL
IMAGE34:EQU     GL+1            ;;COPY OF EXTERNAL CONTROL BITS
CHARS:  EQU     GL+2            ;;NUMBER OF CHARACTERS IN BUFFER
CHAR1:  EQU     GL+3            ;;FIRST CHARACTER RECEIVED FOR BAUD RATE
RXHOLD: EQU     GL+4            ;;RECEIVER HOLDING REGISTER
TXHOLD: EQU     GL+5            ;;TRANSMITTER HOLDING REGISTER
CT1200: EQU     GL+6            ;;COUNTER, DECREMENTED AT 1200 HZ RATE
IRQ3PTR:EQU     GL+7
IRQ4PTR:EQU     GL+8
BITCOUNT: EQU   GL+9            ;;BIT COUNTER FOR RX UART ROUTINE
LCHARS: EQU     GL+10           ;;LENGTH OF LAST BUFFER (FOR REPEAT)
TXDATA: EQU     GL+11
TXFLAG: EQU     GL+12           ;;FF=NORMAL XMIT CONTROL, (NOT TXON)=NO CARRIER
RESULTS: EQU    GL+13           ;;RESULT CODE DIGIT 0..5
BUFPNTR: EQU    GL+14           ;;RELATIVE BUFFER POINTER FOR COMMANDS
;;
FLAGS:  EQU     252             ;;Z8 STATUS FLAG REGISTER
ADDRESS: EQU    0FEH            ;;USED BY "S" COMMANDS
;;
MAXCHARS:       EQU     40      ;;LENGTH OF INPUT BUFFER
;;
;;MAP OF IMAGE35:
SWHOOK: EQU     1               ;;1=OFF HOOK
RJ13:   EQU     2               ;;1=RJ13 AUX CONTACTS CLOSED
AA$LED: EQU     4               ;;0=AA LED ON
CON2:   EQU     8               ;;1= DATA TO SLAVE CPU STABLE
HS$LED: EQU     10H             ;;0=HS LED ON
RING:   EQU     20H             ;;0=RING INDICATOR TRUE
CARRIER:EQU     40H             ;;0=CARRIER DETECT TRUE
SHUNT:  EQU     80H             ;;1=SHORT LINE SIDE OF TELCO TRANSFORMER
;;
;;MAP OF IMAGE 34
;;
SQRQ:   EQU     1               ;;NO CURRENT FUNCTION
PSK:    EQU     2               ;;1=PSK MODE  0=FSK MODE
ORIG:   EQU     4               ;;1=ORIGINATE MODE
BELL:   EQU     8               ;;1=BELL 212  0=CCITT V.22
ALB:    EQU     10H             ;;1=ANALOG LOOP BACK
TXFBP:  EQU     20H             ;;1=BYPASS TX FILTER FOR TONE DIALING
VOL0:   EQU     40H             ;;VOLUME CONTROL BITS
VOL1:   EQU     80H             ;;//
;;
;;
;;INTERRUPT VECTORS
;;
        DW      XIRET           ;;IRQ0 P3-2 STATUS FROM SLAVE CPU 
        DW      XIRET           ;;IRQ1 P3-3 INPUT FROM DTR LINE
        DW      DCLOCK          ;;IRQ2 P3-1 CLOCK (1200HZ) (DATA SAMPLED)
        DW      IRQ3            ;;IRQ3 P3-0 TX DATA
        DW      IRQ4            ;;IRQ4 VECTOR FOR TIMER 0
        DW      IRQ5            ;;IRQ5 VECTOR FOR TIMER 1
;;
        JP      START           ;;GO FOR IT!!
;;
IRQ5:   JP      @IRQ5VH         ;;MAIN TIME BASE ROUTINE
;;
;;
STATE0: LD      R3,PFLAGS3
        AND     R3,#SBREAK      ;;GET BREAK AND ALTERNATING 1 AND 0 BITS
        SWAP    R3              ;; // TO POSITIONS 6,7
        XOR     R6,R3           ;;XOR WITH BITS 15,16 OF SCRAMBLER      
        LD      R3,#STATE12&255 ;;/ WHICH INVERTS THE NEXT ONE OR TWO DATA BITS
        LD      R8,#3           ;;COUNT 2 DEAD STATES NEXT
        SRP     PRIMREG 
XIRET:  IRET
;;
STATE12: DJNZ   R8,S3DONE       ;;DO NOTHING FOR 2 STATES
;;
STATE3: LD      R3,#STATE4&255
ENTRS3: OR      CT1200,CT1200           ;;TEST COUNTER FOR ZERO
        JR      Z,S3DONE
        DEC     CT1200                  ;;DECREMENT IF NOT ZERO
S3DONE: SRP     PRIMREG
        IRET
;;
STATE4: 
        LD      R3,#STATE5&255          ;;ASSUME STATE5 NEXT
        TM      IMAGE35,#CARRIER        ;;CHECK FOR CARRIER
        JR      NZ,ST4X                 ;;NO DLB IF NO CARRIER
        TM      LPBACK,#S16DLB+S16RDL   ;;TEST FOR DIGITAL LOOP BACK MODES
        JR      Z,ST4X                  ;;JUMP IF NORMAL MODE
        LD      R3,#STATE5A & 255       ;;IF DLB THEN DO STATE5A INSTEAD
ST4X:   SRP     PRIMREG
        IRET    
;;
;;DATA SCRAMBLER ROUTINE.   INTERRUPT REGISTERS R5,R6 & R7
;; FORM A 17 BIT SHIFT REGISTER.  R5 IS MSB, R7 IS LSB
;;
STATE5: ;;THIS ROUTINE EXECUTED ON BOTH STATE 5 AND STATE D.
        LD      R3,#STATE6789A&255
        LD      R8,#6           ;;COUNT 5 DEAD STATES NEXT
ENTR5:  RRC     R5              ;; BIT 17 OF SCRAMBLER INTO CARRY
        TM      R6,#20H         ;;TEST BIT 14
        JR      Z,SC0
        CCF                     ;;EXOR 17 AND 14
SC0:    TM      TXDATA,#1       ;;TEST DATA BIT
SC1:    JR      Z,SC2
        CCF                     ;;EXOR WITH DATA
SC2:    ADC     R7,R7           ;;SHIFT THE SCRAMBLER REGISTER
        ADC     R6,R6
        ADC     R5,R5
        SRP     PRIMREG
        IRET
;;STATES 5A,6A,7A,8A,DA EXECUTED ONLY DURING DIGITAL LOOP BACK------
STATE5A:
        LD      R3,#STATE6A&255 ;;THIS IS FOR DIGITAL LOOP BACK MODE
ENTR5A: RRC     R5
        TM      R6,#20H
        JR      Z,SC05A
        CCF
SC05A:  TM      PORT2,#RDATA    ;;SAMPLE RECEIVE DATA FOR DIGITAL LOOPBACK
        JR      SC1
;;      
;;
STATE6A:LD      R3,#STATE7A&255
        TM      PORT3,#CLOCK    ;;LOOK AT CLOCK LEVEL
        JR      NZ,STATE7A      ;;SKIP NEXT STATE IF PHASE NOT CORRECT
ST6X:   SRP     PRIMREG
        IRET
;;
STATE7A: LD     R3,#STATE8A&255
        JR      ST8X
;;
;;
STATE8A:LD      R3,#STATE6789A & 255
        LD      R8,#3           ;;2 NOP STATES NEXT (9,A)
        TM      PORT3,#CLOCK    ;;LOOK AT CLOCK LEVEL
        JR      Z,STATE7A       ;;HOLD HERE TILL PHASE IS CORRECT
ST8X:   SRP     PRIMREG
        IRET
;;-----------------------------------
;;THIS TRICK DOES STATES 6,7,8,9,A.  R8 WAS LOADED WITH 5 BACK AT STATE 5.
;;
STATE6789A:
        DJNZ    R8,ST8X
;;--------------------------------------
STATEB: LD      R3,#STATEC&255
        JR      ENTRS3                  ;;1200 HZ COUNTER
;;
STATEC: LD      R3,#STATED&255          ;;ASSUME STATE-D NEXT
        TM      IMAGE35,#CARRIER        ;;CHECK FOR CARRIER
        JR      NZ,STCX                 ;;NO DLB IF NO CARRIER
        TM      LPBACK,#S16DLB+S16RDL   ;;TEST FOR DIGITAL LOOP BACK MODE
        JR      Z,STCX                  ;;JUMP IF NORMAL MODE
        LD      R3,#STATEDA & 255       ;;IF DLB THEN DO STATE-DA INSTEAD
STCX:   SRP     PRIMREG
        IRET    
        
;;
STATED: LD      R3,#STATEE&255
        JR      ENTR5                   ;;NORMAL MODE
;;
STATEDA: LD     R3,#STATEE&255
        JR      ENTR5A                  ;;DIGITAL LOOP BACK MODE
;;
STATEE: LD      R3,#STATEF&255
        TM      PFLAGS3,#MOD$DIS        ;;CHECK FOR MODULATION DISABLE BIT
        JR      Z,STEX                  ;;JUMP IF CLEAR
        LD       R3,#STFX               ;;ELSE AVOID MODULATION OF CARRIER
STEX:   SRP     PRIMREG
        IRET    
;;
STATEF: 
        LD      R3,R7                   ;;GET DIBITS FROM SCRAMBLER
        OR      R3,PFLAGS3              ;;FORCE UNSCRAMBLED MARKS IF NEEDED
        AND     R3,#3                   ;;KEEP LOW 2 BITS
        ADD     R3,#DBENC &255          ;;R2,R3 =POINTER TO DIBIT TABLE
        LDC     R3,@RR2                 ;;GET PHASE SHIFT TO R12
        ADD     R12,R3                  ;;ADD PHASE SHIFT TO SINE POINTER
STFX:   LD      R3,#STATE0 & 255        ;;RESET THE STATE POINTER
        SRP     PRIMREG
        IRET
;;
;;
;;

;;
;;------------------------------------------------------
;;212A DIBIT ENCODING TABLE
;; RIGHT MOST BIT SENT FIRST.
DBENC:
        DB      8*N     ;;DIBIT 00 = +90 DEG.
        DB      0       ;;DIBIT 01 = 0
        DB      16*N    ;;DIBIT 10 = 180
        DB      24*N    ;;DIBIT 11 = -90 OR +270
;;

;;------------------------------------------------------
;;
MOREINIT:
        CLR     IMAGE35
        CALL    SHOFF           ;;SET UP IMAGE34,35 FOR ON-HOOK CONDITION
OK:     LD      RESULTS,#'0'            ;;"OK" RESULT CODE
GETCOMM:
        DI
        AND     PFLAGS,#"ONLINE         ;;CLEAR ONLINE FLAG
        LD      SPL,#STACK              ;;SET STACK POINTER
        SRP     PRIMREG
        LD      IMR,#20H                ;;ONLY MASTER CLOCK INTERRUPT
        AND     TMR,#"3                 ;;STOP TIMER T0
        LD      PORT3,#0F0H             ;;SET RS232 DATA OUT TO MARK
        AND     IRQ,#20H                ;;CLEAR ALL OTHER INTERRUPT REQUEST
        EI
        CLR     IRQ3PTR                 ;;CLEAR START BIT INTERRUPT POINTER
        CLR     IRQ4PTR                 ;;CLEAR TIMER 0 INT. POINTER
        LD      LCHARS,CHARS            ;;SAVE LAST BUFFER LENGTH
        LD      CHARS,#-2
        AND     USTAT,#"(IBF+OVL)       ;;CLEAR BUFFER FULL FLAG
        CALL    CLRRI                   ;;CLEAR RING INDICATOR FLAG
        TM      PFLAGS,#IBAUD0+IBAUD1   ;;IS BAUD RATE KNOWN?
        JR      Z,TU0                   ;;DONT SEND RESULT IF BOTH BITS ZERO
        CALL    RESULT                  ;;SEND RESULT CODE OF LAST COMMAND
TU0:    DI
        OR      IMR,#20H+8              ;;ENABLE UART RECEIVER
        EI
LOOP:   TM      IMAGE35,#SWHOOK         ;;TEST FOR OFF-HOOK CONDITION
        JR      NZ,TU2E                 ;;DONT DO RING STUFF IF OFF-HOOK
        TM      PORT3,#STATUS           ;;CHECK FOR RING SIGNAL
        JR      Z,TU2B                  ;;JUMP IF NO RING
        TM      IMAGE35,#RING           ;;SEE IF RING LED ALREADY ON
        JR      Z,TU2E                  ;;IF ON DONT WASTE TIME DOING IT AGAIN
        CALL    SETRI                   ;;SET RI AND AA-LED IF NOT ON
        JR      TU2E                    ;;THEN CONTINUE
;;
TU2B:   TM      IMAGE35,#RING           ;;SEE IF RING WAS TRUE BEFORE
        JR      NZ,TU2D                 ;;DONT DO ANY THING IF RING FALSE
        INC     RINGS                   ;;ELSE BUMP RING COUNTER "S1" REGISTER
        CALL    CLRRI                   ;;  AND CLEAR RI AND AA-LED
        LD      RESULTS,#'2'            ;;GET RING RESULT CODE
        DI
        CP      IRQ3PTR,#0
        JR      NZ,TU2C                 ;;DONT SEND RING IF IN COMMAND LINE
        PUSH    IMR                     ;;SAVE INTERRUPT MASK REGISTER
        AND     IMR,#"18H               ;;KILL COMMAND PROCESSOR INTERRUPTS
        EI
        CALL    RESULT                  ;;SEND RING RESULT
        DI                              ;;     /
        POP     IMR                     ;;RE-ENABLE COMMAND PROCESSOR
TU2C:   TM      PORT3,#DTR              ;;  /TEST FOR TRUE RS232 DTR LEAD
        EI                              ;; /
        JR      Z,TU2D                  ;;/ JUMP IF DTR FALSE (IGNORE RING)
        CP      RINGN,#0                ;;SEE IF SET TO IGNORE RINGS
        JR      EQ,TU2D                 ;;JUMP IF WE IGNORE THE RING
        CP      RINGS,RINGN             ;;COMPARE RINGS TO STORED VALUE
        JP      EQ,A$COMM               ;; GO ONLINE IN ANSWER MODE IF MATCH
;;
TU2D:   TM      PORT2,#RDATA            ;;SEE IF 8 SECONDS WITH NO RING PASSED
        JR      NZ,TU2E                 ;;JUMP IF RING DETECTED IN LAST 8 SEC.
        CLR     RINGS                   ;;ELSE CLEAR RING COUNTER (S1)
TU2E:   TM      OFLAGS,#LEON            ;;TEST FOR LOCAL ECHO
        JR      Z,TU3                   ;;JUMP IF NO LOCAL ECHO
        LD      R0,PORT3                ;;GET INPUT RS232 DATA (LSB)
        RR      R0                      ;;MOVE LSB TO MSB
        OR      R0,#70H
        LD      PORT3,R0                ;;ECHO THE BIT
TU3:    TM      PFLAGS,#CONEST          ;;SEE IF WE ARE ON THE PHONE LINE
        JR      Z,TU4A                  ;;JUMP IF NOT
        CLR     PORT0                   ;;ADDRESS THE DIRECT CARRIER DET. BIT
        CP      LCDT,#255               ;;SHOULD WE IGNORE CARRIER DETECT?
        JR      EQ,TU4A                 ;;  / YES, JUMP 
        TM      PORT2,#MUXOUT           ;;TEST MUX OUTPUT BIT
        JR      Z,TU4A                  ;;JUMP IF CARRIER TRUE (LOW)
        CALL    SHOFF                   ;; /  ELSE HANGUP WHEN CARRIER LOST
TU4A:
        TM      USTAT,#RRF              ;;CHECK FOR CHARACTER
        JR      Z,TU4                   ;;JUMP IF NO CHARACTER PRESENT
        AND     USTAT,#"RRF             ;;ELSE CLEAR RRF FLAG
        CP      RXHOLD,BSC              ;;SEE IF BACK SPACE CHAR
        JR      NZ,TU4                  ;;JUMP IF NOT BACK SPACE
        CP      CHARS,#0                ;;ARE WE AT BEGINNING OF BUFFER?
        JR      LE,TU4                  ;;DONT BACK UP PAST BEGINNING
        DEC     CHARS                   ;;NOW BACK UP CHARACTER COUNTER
        LD      TXHOLD,#20H             ;;SEND A SPACE
        CALL    CHAROUT
        LD      TXHOLD,BSC              ;;THEN A BACK SPACE
        CALL    CHAROUT
TU4:
        TM      USTAT,#IBF              ;;INPUT BUFFER FULL?
        JP      Z,LOOP
        OR      IMAGE35,#HS$LED         ;;ASSUME LOW SPEED
        TCM     PFLAGS,#OBAUD0+OBAUD1   ;;TEST FOR 1200 BAUD
        JR      NZ,TU5
        XOR     IMAGE35,#HS$LED         ;;TURN ON HIGH SPEED LED
TU5:    CALL    UPDATE35
        CLR     BUFPNTR
;;
;;FALL INTO "EXECUTE" BELOW TO DO THE COMMANDS.
;;--------------------------------------------------------------------
;;              ** COMMAND DECODER **
;;
EXECUTE:LD      SPL,#STACK              ;;RESET STACK POINTER
        LD      RESULTS,#'4'            ;;ASSUME ERROR RESULT
        TCM     PFLAGS,#IBAUD0+IBAUD1   ;;TEST FOR 1200 BAUD INPUT
        JR      NZ,EX00                 ;;JUMP IF LOW SPEED
        TCM     USTAT,#PE+DBITS         ;;TEST FOR 8 DATA BITS PLUS PARITY
        JR      Z,TRICK2                ;;ERROR! 1200 BAUD CANT HAVE 11 BITS
EX00:   TM      USTAT,#OVL              ;;CHECK FOR BUFFER OVERFLOW
        JP      NZ,GETCOMM              ;;QUIT NOW IF OVERFLOW
        CALL    NEXTCHR                 ;;GET NEXT CHARACTER IN BUFFER
        JP      C,OK                    ;;ALL DONE NOW IF CARRY SET
EX0:    LD      R8,#CMDTBL > 8
        LD      R9,#CMDTBL & 255        ;;POINT TO COMMAND TABLE
EX1:    LD      R3,#PRIMREG+5           ;;POINT TO R5
        LDCI    @R3,@RR8                ;;GET ASCII CHARACTER FROM TABLE TO R5
        LDCI    @R3,@RR8                ;;GET HIGH BYTE OF ADDRESS TO R6
        LDCI    @R3,@RR8                ;;GET LOW ADDRESS BYTE TO R7
        CP      R5,#';'                 ;;END OF TABLE MARKER IS ";"
TRICK2: JP      EQ,GETCOMM              ;;QUIT IF AT END WITH ERROR MSG
        CP      R5,R4                   ;;DO WE HAVE A MATCH?
        JR      NE,EX1                  ;;JUMP BACK IF NOT
FOUND:  CALL    @RR6                    ;;CALL THE ROUTINE
        JR      EXECUTE                 ;;LOOP BACK FOR MORE COMMANDS
;;
;;
NEXTCHR:                        ;;RETURNS NEXT BUFFER CHR IN R4
                                ;;CARRY SET IF PAST END OF BUFFER
        LD      R5,BUFPNTR              ;;GET RELATIVE BUFFER POINTER
        LDRX    R4,R5,BUFFER            ;;GET THE DATA FROM THE BUFFER
        CP      CHARS,BUFPNTR           ;;AT END OF BUFFER??
        SCF
        JR      EQ,RETURN
        INC     BUFPNTR                 ;;ADVANCE BUFFER POINTER
        RCF
RETURN: RET
;;
;;
NEXTNUM:                        ;;CONVERTS NEXT DIGITS IN BUFFER
                                ;;TO BINARY VALUE IN R4
        CLR     R6
        CALL    NEXTCHR         ;;GET A DIGIT (THIS IS A FAKE OUT)
        JR      C,LL011         ;;QUIT IF END OF BUFFER
        DEC     BUFPNTR         ;;CORRECT BUFFER POINTER
LL008:  CALL    NEXTCHR         ;;REALLY GET A DIGIT NOW
        JR      C,LL010         ;;QUIT IF END OF BUFFER
        SUB     R4,#'0'         ;;REMOVE ASCII BIAS
        JR      MI,LL009        ;;JUMP IF LESS THAN ZERO
        CP      R4,#10          ;;TEST FOR GREATER THAN 9
        JR      UGE,LL009       ;;QUIT IF NOT DIGIT
R6X10:  LD      R5,R6           ;;MPY R6 TIMES 10
        ADD     R6,R6
        ADD     R6,R6
        ADD     R6,R6
        ADD     R6,R5
        ADD     R6,R5
        ADD     R6,R4           ;;BINARY NUMBER IN R6
        JR      LL008           ;;LOOP BACK FOR MORE DIGITS
LL009:  DEC     BUFPNTR         ;;BACK UP POINTER
LL010:  RCF                     ;;CLEAR CARRY
LL011:  LD      R4,R6           ;;NUMBER IS IN R4 NOW
        OR      R4,R4           ;;SET FLAGS
        RET
;;
;;              ** END OF COMMAND DECODER **
;;
;;------THIS IS THE "A" COMMAND-----------------------------
;;
A$COMM: CALL    W120                    ;;DELAY 120 MS
        AND     PFLAGS,#"MODE           ;;SET TO ANSWER MODE
        AND     IRQ,#"8                 ;;KILL PENDING KEYPRESS
        CALL    SHON                    ;;GO OFF HOOK AND WAIT 2 SEC.
;;
;; FALL INTO "GO$ONLINE" BELOW
;;
;;-------THIS IS THE "O" COMMAND-----------------------------
;;
GO$ONLINE:
        DI
        LD      SPL,#STACK
        LD      IMR,#20H
        CLR     IRQ4PTR
        CLR     IRQ3PTR
        CLR     RINGS
        EI
        TM      OFLAGS,#BELLOPT         ;;TEST FOR BELL OR CCITT MODE
        JR      NZ,OK2GOL               ;;JUMP IF NOT CCITT
        LD      RESULTS,#'4'            ;;GET ERROR RESULT JUST IN CASE
        TCM     PFLAGS,#OBAUD0+OBAUD1   ;;TEST FOR 1200 BAUD TERMINAL
        JP      NZ,ABORT                ;;ABORT WITH ERROR IF CCITT & 300 BAUD
OK2GOL: TM      PFLAGS,#CONEST          ;;SEE IF WE ALREADY ARE ONLINE
        JR      NZ,GOL1                 ;;JUMP IF COMPLETE HANDSHAKE NOT REQ.
GOL0:   TM      PFLAGS,#MODE            ;;TEST FOR ORIG/ANSW MODE
        JP      Z,ANSWR                 ;;FULL ANSWER HANDSHAKE IF ZERO
        TM      PFLAGS2,#BYCXR          ;;LOOK FOR BUSY OR CARRIER?
        JR      Z,SKIPCP                ;;IF NOT THEN SKIP CALL PROGRESS
        TM      LPBACK,#1               ;;TEST FOR ANALOG LOOP BACK
        JR      NZ,SKIPCP               ;;NO CALL PROGRESS IF ALB MODE
        LD      R5,#CPAT+CPBY
        CALL    CALL$PROG
        LD      RESULTS,#'3'            ;;GET NO CARRIER RESULT CODE
        CALL    BTABORT                 ;;ABORT IF BUSY OR TIMEOUT
SKIPCP: TCM     PFLAGS,#(OBAUD0+OBAUD1)
        JP      NZ,ORIG$LOW             ;;LOW SPEED ORIGINATE
        JP      ORIG$HIGH               ;;HIGH SPEED ORIGINATE
;;
GOL1:   
        TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;CHECK BAUD RATE
        JR      Z,GOL2                  ;;JUMP IF HIGH SPEED
        TM      PFLAGS,#MODE            ;;CHECK ORIG/ANSW MODE
        JR      Z,GOL3                  ;;JUMP IF ANSWER
        CALL    LSRXIO                  ;;CALL LOW SPEED RECV. ORIG INIT
        JP      ORIG$LOW2               ;;THEN GO ON LINE
GOL3:   CALL    LSRXIA                  ;;CALL LOW SPEED RECV. ANSW INIT
        JP      ANSWR2                  ;;THEN GO ON LINE
GOL2:   CALL    NEXTNUM                 ;;GET POSSIBLE DIGIT FOLLOWING "O"
        JR      Z,GOL6                  ;;ZERO IS NORMAL MODE
        CP      R4,#2                   ;;2=REMOTE DIGITAL LOOP REQUEST
        JR      ULT,GOL7                ;;JUMP IF LESS THAN 2 BUT > 0
        OR      PFLAGS3,#SUSM           ;;SEND UNSCRAMBLED MARKS (RDL REQUEST)
GOL6:   LD      RESULTS,#'5'            ;;CONNECT 1200 RESULT
        TM      PFLAGS,#MODE            ;;CHECK MODE
        JR      Z,GOL4                  ;;JUMP IF HIGH SPEED ANSWER
        CALL    HSRXIO                  ;;CALL HIGH SPEED RECV ORIG INIT
        JR      GOL5                    ;;GO ON LINE
GOL4:   CALL    HSRXIA                  ;;CALL HIGH SPEED RECV ANSW INIT
GOL5:   JP      AWAIT17                 ;;GO ON LINE
;;
;;"O1" COMMAND...
;;
GOL7:   AND     PFLAGS3,#"TXON          ;;TRANSMITTER OFF
        LD      CT1200,#92              ;;SET FOR 77 MS DELAY
        CALL    WAIT                    ;;WAIT 77 MS
        OR      PFLAGS3,#TXON           ;;TRANSMITTER BACK ON AGAIN
        JR      GOL6
;;
;;
;;-------SPEAKER MODE CONTROL COMMANDS  M0,M1,M2 ------------------
;;
SPCRL:  CALL    NEXTNUM                 ;;GET THE 0,1,2
        RR      R4                      ;;GET BITS IN RIGHT PLACE
        RR      R4
        AND     PFLAGS3,#"(SPON+SPON2)  ;;CLEAR THE DESTINATION
        AND     R4,#(SPON+SPON2)        ;;KEEP ONLY SPEAKER CONTROL BITS
        OR      PFLAGS3,R4              ;;PUT THEM IN "OFLAGS"
        JR      LOUD1
;;
;;------SPEAKER LOUDNESS CONTROL L1,L2,L3 ---------------------
;;
LOUDNESS:
        CALL    NEXTNUM                 ;;GET 1,2,3
        RR      R4
        RR      R4                      ;;GET BITS TO MSB
        AND     R4,#0C0H
        JR      Z,LOUD2                 ;;DONT ALLOW ZERO VOLUME
        AND     PFLAGS2,#"0C0H
        OR      PFLAGS2,R4              ;;SET NEW VOLUME SETTING IN PFLAGS2
LOUD1:  TM      IMAGE35,#SWHOOK         ;;SEE IF WE ARE OFF-HOOK
        JP      NZ,SPKCRL               ;;DO SPEAKER CONTROL IF OFF HOOK
LOUD2:  RET
;;
;;---------------THIS IS THE "=" COMMAND-----------------------
;;
SETRAM: CP      ADDRESS,#(RINGN+18)
        JP      UGE,GETCOMM             ;;ADDRESS TOO BIG, ERROR
        CP      ADDRESS,#RINGN
        JP      ULT,GETCOMM             ;;ADDRESS TOO SMALL, ERROR
;;                                      ;;FALL INTO "ANYLOC" BELOW IF OK
;;
;;---------------THIS IS THE ":" COMMAND------------------------
;;
ANYLOC: CALL    NEXTNUM                 ;;GET DATA;;
        LD      @ADDRESS,R4             ;;PUT IN RAM
        RET                             ;;DONE;;
;;
;;---------------THIS IS THE "S" COMMAND------------------------
;;
SETADR: CALL    NEXTNUM                 ;;GET ADDRESS OF S REGISTER
        ADD     R4,#RINGN               ;;ADD OFFSET TO ADDRESS
        LD      ADDRESS,R4
        RET
;;---------------THIS IS THE " ? " COMMAND------------------------
GETRAM:
GETANY: CP      BUFPNTR,#1              ;;IS "?" FIRST COMMAND IN LINE?
        JP      EQ,HELP0                ;;YES, DO HELP REQUEST
        CALL    WAIT250                 ;;"?" COMMAND <-----
        JR      Z,BDEC                  ;;NO LEADING CRLF IF NON VERBOSE;;
        CALL    CRLF
BDEC:   LD      R4,@ADDRESS             ;;GET DATA BYTE
BDEC2:  LD      R7,#P10TAB&0FFH         ;;POINT TO POWER OF 10 TABLE;;
        LD      R6,#P10TAB>8
LL003:  LDE     R5,@RR6                 ;;GET A POWER OF TEN;;
        INC     R7                      ;;POINT TO NEXT POWER;;
        LD      TXHOLD,#'0'-1
LL004:  INC     TXHOLD                  ;;COUNT POWERS OF TEN;;
        SUB     R4,R5
        JR      NC,LL004
        CALL    CHAROUT                 ;;PRINT ASCII DIGIT;;
        ADD     R4,R5                   ;;RESTORE TO PLUS;;
        CP      R5,#1
        JR      NE,LL003                ;;LOOP FOR ALL POWERS;;
        JP      CRLF                  ;;DONE, DO END OF LINE STUFF;;
;;----------------------------------------------------------------
;;
LS$DISC:
        LD      R7,#LSD                 ;;"Y0,Y1" COMMANDS  <----
        JR      DO$OFLAGS
;;
BELL$CCITT:
        LD      R7,#BELLOPT             ;;"B0,B1" COMMANDS  <----
        JR      DO$OFLAGS
SETECHO:
        LD      R7,#LEON                ;;"E0,E1" COMMANDS  <----
        JR      DO$OFLAGS
;;
SETVERBOSE:
        LD      R7,#VBON                ;;"V0,V1" COMMANDS  <----
        JR      DO$OFLAGS
;;
 
SETFULL:
        LD      R7,#DPLX                ;;"F0,F1" COMMANDS <----
        LD      R8,#PFLAGS
        JR      SETFLAGS
;;
SETQUIET:
        LD      R7,#QUIET               ;;"Q0,Q1" COMMANDS <----
DO$OFLAGS: LD      R8,#OFLAGS
;;
SETFLAGS:
        LD      R9,@R8                  ;;GET THE BYTE TO MODIFY
        OR      R9,R7                   ;;SET MASK FIRST
        CALL    NEXTNUM                 ;;GET THE 0 OR 1 FROM BUFFER
        JR      NZ,LL066                ;;IF NOT ZERO THEN QUIT NOW
        COM     R7                      ;;ELSE INVERT R7 MASK
        AND     R9,R7                   ;;AND CLEAR THE BITS
LL066:  LD      @R8,R9                  ;;PUT  THE BYTE BACK
        RET
;;
;;---------THIS IS THE "X" COMMAND--------------------------
;;
EXRESULT:
        CALL    NEXTNUM                 ;;GET X PARAMETER
        CP      R4,#4                   ;;CHECK RANGE
        JP      UGT,GETCOMM             ;;IF OUT THEN EXIT WITH ERROR MSG
        AND     PFLAGS2,#"(WDT+BYCXR+EXR)  ;;CLEAR STATUS BITS
        LD      R8,#XTABLE > 8          ;;INDEX INTO TABLE
        LD      R9,R4
        LDC     R4,@RR8                 ;;GET STATUS BIT MASK
        OR      PFLAGS2,R4              ;;PUT BITS INTO PFLAGS2
        RET     
;;
;;
;;---------THIS IS THE "R" COMMAND-----------------
;;
REVERSE:
        XOR     PFLAGS,#MODE            ;;"R" COMMAND  <----
        RET
;;
;;---------"P" COMMAND---------
ENABPULSE:
        AND     PFLAGS2,#"TDIAL         ;;"P" COMMAND  <----
        RET
;; 
ENABTONE:
        OR      PFLAGS2,#TDIAL          ;;"T" COMMAND  <----
        RET
;;
;;--------------THIS IS THE "W" COMMAND-----------------
;;
;;
WAITDIAL: LD    R5,#CPDT2+CPBY          ;;LOOK FOR SECOND TONE OR BUSY
WTDT1:  CALL    CALL$PROG
        LD      RESULTS,#'6'            ;;NO DIALTONE RESULT CODE
BTABORT:CP      R5,#CPTIME              ;;TIMEOUT?
        JP      EQ,ABORT                ;;JUMP IF NO DIAL TONE
        LD      RESULTS,#'7'            ;;BUSY RESULT
        CP      R5,#CPBY
        JP      EQ,ABORT                ;;ABORT IF BUSY
        LD      RESULTS,#'4'
TPRET:  RET                             ;;RETURN IF DIAL TONE FOUND
;;
FIRSTDIAL:
        LD      R5,#CPDT1               ;;LOOK FOR FIRST DIAL TONE ONLY
        JR      WTDT1
;;
;;-------------------------------------------------------------------
;;THIS IS THE PRODUCT ID CODE GENERATOR  (THE "I" COMMAND)
;;ALSO, "I1" WILL RETURN THE CHECKSUM OF THE PROM
;;
PRODUCT$ID:
        CALL    WAIT250                 ;;DELAY AND GET VERBOSE FLAG
        JR      Z,PID0                  ;;NO LEADING CRLF IF NON-VERBOSE
        CALL    CRLF                    ;;CRLF IF VERBOSE
PID0:   CALL    NEXTNUM                 ;;GET 0 OR 1, AND MAYBE SET ZERO FLAG
        LD      R0,R4                   ;;SAVE DIGIT IN R0
        LD      R4,#VERS                ;;GET PRODUCT ID/ VERSION NUMBER
        JP      Z,BDEC2                 ;;JUMP IF PRODUCT ID REQUEST
MEMTST: CLR     R6                      ;;POINT TO FIRST LOCATION
        CLR     R7                      ;;  /
        CLR     R4                      ;;CLEAR CHECKSUM
PID1:   LDE     R5,@RR6                 ;;GET A DATA ROM BYTE
        ADD     R4,R5                   ;;ADD IT TO CHECKSUM
        LDC     R5,@RR6                 ;;GET CODE ROM BYTE
        ADD     R4,R5                   ;;ADD IT CHECKSUM
        INCW    RR6                     ;;POINT TO NEXT ADDRESS 
        CP      R6,#10H                 ;;1000H ?               
        JR      NE,PID1                 ;;LOOP TIL IT GOES TO 1000H
        CP      R0,#1                   ;;CHECK FOR I1 COMMAND
        JP      EQ,BDEC2                ;;PRINT CHECKSUM IF I1
        CP      R4,#0                   ;;CHECKSUM SHOULD ALWAYS BE ZERO 
        JP      NZ,GETCOMM              ;;ERROR MSG IF NOT CORRECT
        RET                             ;;ELSE RETURN OK
;;
;;--------------------------------------------------------------------
;;
;;THIS IS THE FLASH COMMAND.  IT ONLY WORKS IF MODEM IS OFF HOOK
;;FLASH LENGTH IS FIXED AT 500 MS.
;;
FLASH:  AND     IMAGE35,#"SWHOOK        ;;GO ON HOOK
        CALL    UPDATE35
        CALL    HALFSEC                 ;;WAIT 1/2 SECOND
        OR      IMAGE35,#SWHOOK         ;;GO BACK OFFHOOK
        CALL    UPDATE35                ;;THEN WAIT ANOTHER 1/2 SEC
HALFSEC: CALL   WAIT250
        CALL    WAIT250
        RET
;;
;;
NMANSW:
        LD      R5,#CPAN+CPBY           ;;BUSY OR NON-MODEM ANSWER
        CALL    CALL$PROG
        LD      RESULTS,#'8'            ;;NO ANSWER RESULT
        CALL    BTABORT                 ;;ABORT IF BUSY OR TIMEOUT
        RET
;;
;;
;;------------------------------------------------------------------
;;
;;CALL PROGRESS MONITOR ROUTINE
;;
;;REGISTER USAGE
;;      R5      TELL SUBROUTINE WHAT WE WANT IT TO LOOK FOR. (BIT MAPPED)
;;              ..AND RETURN STATUS OF WHICH SIGNAL WAS FOUND
;;              BIT 0   SECOND DIAL TONE (3 SEC./ 30 SEC. TIMEOUT)
;;              BIT 1   BUSY
;;              BIT 2   RING
;;              BIT 3   ANSWER
;;              BIT 4   MODEM ANSWER TONE
;;              BIT 5   FIRST DIAL TONE   (1 SEC./ 5 SEC. TIMEOUT)
;;      R4      BUSY COUNTER
;;      R6      SCRATCH
;;      R7      TIMEOUT VALUE (SECONDS) 
;;      R8      CURRENT TONE LENGTH (N*70MS)
;;      R9      LENGTH OF SPACE BETWEEN TONES (N*70MS)
;;      R10     LAST TONE LENGTH  (N*70 MS)
;;
;;R5 BITMAP
CPTIME: EQU     0               ;;TIMEOUT, NO VALID TONES DETECTED
CPDT2:  EQU     1               ;;SECOND DIAL TONE
CPBY:   EQU     2               ;;BUSY
CPRI:   EQU     4               ;;RING BACK
CPAN:   EQU     8               ;;ANSWER
CPAT:   EQU     10H             ;;MODEM ANSWER TONE
CPDT1:  EQU     20H             ;;FIRST DIAL TONE
;;
;;CHECK FOR A BUSY SIGNAL
;;
CPM4:   CP      R10,#2          ;;CHECK LENGTH OF LAST TONE
        JR      ULT,CPM5        ;;IGNORE TONES LESS THAN 140 MS
        CP      R8,#2
        JR      ULT,CPM5
        CP      R10,#11         ;;CHECK LAST TONE LENGTH
        JR      UGE,CPM5        ;;NOT BUSY IF MORE THAN 770 MS
        CP      R8,#11          ;;CHECK CURRENT TONE FOR LESS THAN 770 MS
        JR      UGE,CPM5        ;;INVALID IF MORE THEN 770
        CP      R9,#11          ;;CHECK FOR LESS THAN 770 MS OF SILENCE
        JR      UGE,CPM5        ;;INVALID IF GREATER THAN 770
        CP      R9,#2           ;;TEST FOR LESS THAN 140 MS OF SILENCE
        JR      ULT,CPM5        ;;INVALID IF LESS THAN 140 MS
        ADD     R10,R9          ;;ADD SILENCE TO TONE LENGTH
        CP      R10,#18         ;;SEE IF SUM IS GREATER THAN 1.3 SEC.
        JR      UGE,CPM5        ;;JUMP IF INVALID BUSY 
        INC     R4              ;;INCREMENT BUSY COUNTER
        TM      R5,#CPBY        ;;SEE IF USER WANTED BUSY ENABLED
        JR      Z,CPM1          ;;JUMP IF NOT
        CP      R4,#3           ;;CHECK FOR 3 BUSY CYCLES
        JR      NE,CPM1         ;;JUMP IF LESS THAN 3
        LD      R5,#CPBY        ;;SET BUSY CODE
        RET
;;
CPM5:   CLR     R4              ;;CLEAR BUSY COUNTER
        JR      CPM1
;;
;;THIS IS THE CALL PROGRESS ENTRY ADDRESS
;;
CALL$PROG:
        LD      R6,#14
        CLR     R4              ;;CLEAR BUSY COUNTER
        CLR     R8              ;;CLEAR TONE LENGTH
        LD      R7,#5           ;;ASSUME 5 SECOND TIMEOUT
        TM      R5,#CPDT1       ;;CHECK FOR FIRST DIAL TONE REQUEST
        JR      NZ,CPM1         ;;KEEP 5 SECOND TIMEOUT IF FIRST DIAL TONE
        LD      R7,CDW          ;;ELSE USE "S7" REGISTER FOR TIMEOUT (30 SEC.)
CPM1:   LD      R10,R8          ;;PUT CURRENT TONE INTO OLD TONE REGISTER
        LD      R8,CDELAY       ;;SET DATA ANSWER TONE LENGTH COUNTER
CPM1A:  CLR     R9              ;;CLEAR SILENCE LENGTH COUNTER
CPM2:   CALL    WAIT70          ;;WAIT 70 MS TILL NEW SAMPLE COMES
        JR      NC,XCPM         ;;EXIT IF TIMEOUT
        TM      PORT2,#RDATA    ;;CHECK FOR TONE OR SILENCE
        JR      NZ,CPM2B        ;;JUMP IF TONE PRESENT
        TM      R5,#CPAT        ;;USER WANTS ANSWER TONE?
        JR      Z,CPM2A         ;;JUMP IF NOT
        TM      PORT3,#STATUS   ;;CHECK FOR ANSWER TONE
        JR      Z,CPM2A
        DJNZ    R8,CPM1A        ;;COUNT 70 MS INTERVAL OF ANSWER TONE
        LD      R5,#CPAT
        RET                     ;;RETURN WITH MODEM ANSWER TONE PRESENT
;;
CPM2A:  LD      R8,CDELAY       ;;RELOAD ANSWER TONE TIMER
        INC     R9              ;;BUMP SILENCE COUNTER
        CP      R9,#72          ;;CHECK FOR >5 SECONDS OF SILENCE
        JR      ULT,CPM2        ;;LOOP BACK IF LESS THEN 5 SEC. SILENCE
        CP      R10,#3          ;;CHECK LENGTH OF LAST TONE
        JR      ULT,CPM2        ;;LOOP BACK IF TONE LESS THEN 210 MS
        TM      R5,#CPAN        ;;SEE IF USER WANTS TO LOOK FOR ANSWER COND.
        JR      Z,CPM2          ;;IF NOT THEN LOOP BACK
        LD      R5,#CPAN        ;;RETURN ANSWER CODE
XCPM:   RET
;;
CPM2B:  CLR     R8              ;;CLEAR TONE LENGTH COUNTER
CPM3:   CALL    WAIT70          ;;WAIT FOR NEW SAMPLE
        JR      NC,XCPM         ;;EXIT IF TIMEOUT
        TM      PORT2,#RDATA    ;;TEST FOR TONE
        JP      Z,CPM4          ;;JUMP IF TONE GONE
        INC     R8              ;;BUMP TONE LENGTH COUNTER
        TM      R5,#CPDT1       ;;LOOK FOR FIRST DIAL TONE??
        JR      Z,CPM3A         ;;JUMP IF NOT
        CP      R8,#14          ;;CHECK FOR 1 SECOND OF TONE
        JR      ULT,CPM3        ;;LOOP BACK IF LESS THEN 1 SEC
        LD      R5,#CPDT1
        RET                     ;;RETURN WITH 1ST DIAL TONE STATUS
CPM3A:  CP      R8,#43          ;;COMPARE WITH 3 SEC. VALUE
        JR      ULT,CPM3        ;;LOOP BACK IF LESS THAN 3 SECONDS
        TM      R5,#CPDT2       ;;SEE IF USER WANTS SECOND DIAL TONE
        JR      Z,CPM3          ;;LOOP BACK IF NOT
        LD      R5,#CPDT2       ;;ELSE SET DIAL TONE CODE
        RET                     ;;AND RETURN
;;SAMPLE THE PHONE LINE NOISES
;;
WAIT70: TM      IRQ,#8          ;;TEST FOR KEY PRESSED
        JP      NZ,ABORT$DIAL   ;;QUIT IF TRUE
        TM      PORT3,#DTR      ;;TEST FOR FALSE DTR LINE
        JP      Z,ABORT$DIAL    ;;QUIT IF FALSE
        TM      IRQ,#4          ;;TEST FOR CLOCK LINE CHANGE
        JR      Z,WAIT70        ;;LOOP HERE FOR 70 MS
        AND     IRQ,#"4         ;;CLEAR FLAG
        DJNZ    R6,W70EXIT      ;;JUMP IF NO TIMEOUT TO TEST FOR TONE
        LD      R6,#14          ;;RELOAD 1 SECOND COUNTER
        DJNZ    R7,W70EXIT      ;;JUMP IF NO TIMEOUT
        CLR     R5              ;;NO STATUS BITS SET FOR TIMEOUT
        RCF                     ;;RESET CARRY AND RETURN IF TIMEOUT
        RET
W70EXIT: SCF
        RET
;;-------------------------------------------------------
;;THIS IS THE DIALING ROUTINE
;;
DIAL:
        CALL    W120                    ;;WAIT 120 MS FOR POSSIBLE LINE FEED
        DI
        LD      SPL,#STACK
        LD      IMR,#20H                ;;ONLY MASTER TIMEBASE INTERRUPTS ENAB.
        AND     IRQ,#"8                 ;;KILL PENDING KEYPRESS FLAG
        EI
        OR      PFLAGS,#MODE            ;;SET FOR ORIGINATE MODE
        CALL    SHON                    ;; GO OFF-HOOK , WAIT FOR DIAL TONE
;;
DLOOP:  CALL    NEXTCHR                 ;;NEXT CHAR. FROM BUFFER INTO R4
        JP      C,GO$ONLINE             ;;GO WAIT FOR CARRIER IF END OF BUFFER
LL058:  LD      R5,#16                  ;;NUMBER OF TONES IN TABLE
        LD      R8,#TONETBL > 8         ;;HI TABLE POINTER
        LD      R9,#TONETBL & 255       ;;LOW TONE TABLE POINTER
LL059:  LDE     R6,@RR8                 ;;GET CHAR FROM TABLE
        INC     R9                      ;;THEN POINT TO ROW/COLLUM DATA
        CP      R4,R6                   ;;MATCH INPUT??
        JR      EQ,FOUND$DIGIT
        INC     R9                      ;;ADVANCE POINTER TO NEXT ENTRY
        DJNZ    R5,LL059                ;;AND LOOP BACK
        LD      R8,#CMDT2 > 8
        LD      R9,#CMDT2 & 255         ;;POINT TO LAST PART OF COMMAND TABLE
LL060:  LD      R3,#PRIMREG+5           ;;R3 POINTS TO R5
        LDCI    @R3,@RR8                ;;GET ASCII CHARACTER FROM TABLE TO R5
        LDCI    @R3,@RR8                ;;GET HIGH BYTE OF ADDRESS TO R6
        LDCI    @R3,@RR8                ;;GET LOW ADDRESS BYTE TO R7
        CP      R5,#0                   ;;CHECK FOR END OF TABLE
        JR      EQ,LL061                ;;IGNORE CHARACTER IF NOT IN TABLE
        CP      R5,R4                   ;;COMMAND MATCH TABLE ENTRY?
        JR      NE,LL060                ;;IF NOT THEN GET NEXT COM. TO COMPARE
        CALL    @RR6                    ;;ELSE DO COMMAND @RR6
LL061:  JR      DLOOP                   ;;BACK TO TOP OF LOOP FOR NEXT COMMAND
 
FOUND$DIGIT:
        DI
        LD      IRQ5VH,#DIALER$TB > 8   ;;POINT VECTORS TO DIALER TIMEBASE
        LD      IRQ5VL,#DIALER$TB & 255 ;; /
        CALL    ZERO$FREQ               ;;TURN OFF TONE GENERATOR
        EI                              ;;ENABLE INTERRUPTS
        TM      PFLAGS2,#TDIAL          ;;SEE IF WE DO TONE DIALING
        JR      Z,PULSEDIAL             ;;JUMP IF PULSE DIALING
        DI
        LD      R0,#FRQTBL > 8          ;;HIGH BYTE OF FREQUENCY LOOK UP TABLE
        LDE     R1,@RR8                 ;;GET POINTERS
        LD      R6,#INTREG+10
        CALL    GETFRQ                  ;;LOAD FREQUENCY INTO INTREG 10,11
        LDE     R1,@RR8                 ;;GET POINTERS AGAIN
        SWAP    R1                      ;;GET HIGH TONE NIBBLE TO LOW HALF
        LD      R6,#PRIMREG+2           ;;POINT TO DESTINATION
        CALL    GETFRQ                  ;;LOAD R2,R3 WITH HIGH GROUP FREQ.
        EI                              ;;TONE STARTS NOW
        CALL    TLWAIT                  ;;WAIT FOR TONE DURATION
        CALL    ZERO$FREQ               ;;TURN OFF THE TONE
        CALL    TLWAIT                  ;;WAIT
DIGDONE:DI
        AND     IMAGE35,#"SHUNT         ;;OPEN SHUNT RELAY
        CALL    UPDATE35
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET FOR MAIN TIME BASE ROUTINE
        LD      IRQ5VL,#TIMEBASE & 255  ;; /
        EI
        JP      DLOOP                   ;;BACK FOR NEXT DIGIT
;;
;;CALL WITH R6 POINTING TO DESTINATION AND TABLE OFFSET IN LOW NIBBLE
;; IN R1
GETFRQ: AND     R1,#0FH                 ;;KEEP LOW GROUP POINTER
        ADD     R1,#FRQTBL & 255        ;;ADD BASE ADDRESS
        LDEI    @R6,@RR0                ;;GET LOW FREQUENCY
        LDEI    @R6,@RR0
        RET
;;
TLWAIT: TM      LPBACK,#2               ;;CHECK FOR LONG TONE TEST MODE
        JR      NZ,TLWAIT               ;;WAIT TILL KEY PRESSED IF TEST MODE
        LD      R1,TONLEN               ;;ELSE GET LENGTH OF TOUCH TONE
LL062:  LD      R0,#9                   ;;PREPARE TO COUNT 10 1.04 MS INTERVALS
LL063:  OR      R0,R0                   ;; TEST R0 FOR ZERO COUNT
        JR      NZ,LL063                ;;WAIT TILL R0 GOES TO ZERO
        DJNZ    R1,LL062                ;;COUNT 1.04 MS PERIODS WITH R1
        RET
;;
;        
;;THIS IS THE PULSE DIALER...  R5 HAS NUMBER OF PULSES
;;
PULSEDIAL:
        CP      R5,#11                  ;;0..9 RANGE?
        JR      UGE,DIGDONE             ;;EXIT IF NOT VALID DIGIT
LL064:  DI
        AND     IMAGE35,#"SWHOOK
        CALL    UPDATE35                ;;GO ON-HOOK
        LD      R1,#62                  ;; 62*1.04167 = 64.5 MS BREAK
        EI
        CALL    LL062                   ;;WAIT....
        DI
        OR      IMAGE35,#SWHOOK+SHUNT   ;;PULL SWITCH AND SHUNT RELAY
        CALL    UPDATE35                ;;GO OFF-HOOK AGAIN
        LD      R1,#31                  ;;66.6/33.3 % RATIO @ 10.3 PPS
        TM      OFLAGS,#MB6139          ;;TEST MAKE/BREAK RATIO OPTION
        JR      Z,LL064A                ;; JUMP IF NOT USA/CANADA
        LD      R1,#39                  ;;61.4/38.6 % RATIO @ 9.5 PPS
LL064A: EI
        CALL    LL062                   ;;WAIT
        DJNZ    R5,LL064                ;;LOOP BACK FOR MORE CLICKS
        LD      R5,#30                  ;;SET UP FOR 750 MS INTERDIGIT DELAY
LL065:  LD      R0,#240                 ;;
WAIT25: OR      R0,R0                   ;;R0 DECREMENTED BY "DIALER$TB"
        JR      NZ,WAIT25               ;;LOOP HERE FOR 25 MS
        DJNZ    R5,LL065                ;;COUNT 30 25 MS INTERVALS FOR 750 MS
        JR      DIGDONE                 ;;FINISHED DIALING 1 DIGIT
;;
;;
ZERO$FREQ:
        AND     PFLAGS3,#"TXON          ;;TRANSMITTER OFF
ZF0:    PUSH    IMR
        DI
        LD      R3,#6
        LD      R2,#INTREG+10
ZF1:    CLR     @R2                     ;;CLEAR INTREG 10-15
        INC     R2
        DJNZ    R3,ZF1
        CLR     R2
        LD      INTREG+2,#STATEF > 8    ;;SET INDIRECT POINTERS
        LD      INTREG+3,#STATEF & 255
        POP     IMR
SHRET:  RET
;;
;;
;;
STOP$DIAL:                              ;;THIS IS THE ";" COMMAND <-----
        DI
        CALL    ZERO$FREQ
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET FOR MAIN TIME BASE ROUTINE
        LD      IRQ5VL,#TIMEBASE & 255  ;; /
        EI
        JP      EXECUTE                 ;;EXIT WITHOUT GOING ONLINE BUT OFFHOOK
;;------------------------------------------------------------------
;;THE "H" COMMAND ENTERS AT "HOOKCRL:"
;;
;;CALL "SHON" TO CONNECT TO PHONE LINE AND "SHOFF" TO DISCONNECT.
;;
HOOKCRL:
        CALL    NEXTNUM                 ;; TEST FOR H0,H1,H2
        JR      Z,SHOFF1                ;;H0 = SWITCH HOOK OFF
        AND     IMAGE35,#"RJ13          ;;ASSUME RJ13 RELAY OFF
        DJNZ    R4,NO$RJ13              ;;IF H2 DONT PULL RJ13 RELAY
        OR      IMAGE35,#RJ13
NO$RJ13: OR     IMAGE35,#SWHOOK         ;;PULL SWITCH HOOK RELAY(S)
        CALL    UPDATE35                ;; /
        JP      SPKCRL                  ;;DO SPEAKER CONTROL
;;
SHON:   OR      IMAGE35,#RJ13           ;;/ ELSE PULL IN RJ13 RELAY
        CALL    SPKCRL                  ;;DO SPEAKER CONTROL
        TM      IMAGE35,#SWHOOK         ;;TEST SWITCH HOOK BIT
        JR      NZ,SHRET                ;;DONT DO HARDWARE UPDATE IF OFF-HOOK
NORJ13: OR      IMAGE35,#SWHOOK         ;;CLOSE SWITCH HOOK RELAY
        CALL    UPDATE35                ;;MOVE IMAGE35 TO EXTERNAL HARDWARE
        LD      R4,#2                   ;;SET FOR 2 SECOND WAIT
        TM      PFLAGS,#MODE            ;;SEE IF MODE IS ANSWER
        JP      Z,WAITSEC               ;;EXIT, WAIT 2 SECONDS IF ANSWER MODE
        LD      IMAGE34,#TXFBP          ;;ELSE SET UP FOR ORIGINATE MODE
        CALL    UPDATE34                ;;// FOR DIALING AND CALL PROGRESS
        LD      R4,DTW                  ;;GET DIAL TONE WAIT TIME
        CP      R4,#2                   ;;SEE IF ITS 2 SEC OR MORE
        JR      UGE,DTWOK               ;;DTW OK IF 2 SEC. OR GREATER
        LD      R4,#2                   ;;IF LESS THEN SET FOR 2 SEC.
DTWOK:  TM      PFLAGS2,#WDT            ;;REAL DIAL TONE WAIT ENABLED
        JP      NZ,FIRSTDIAL            ;;IF TRUE THEN WAIT FOR REAL DIAL TONE
        JP      WAITSEC                 ;; / ELSE JUST WAIT "DTW" SECONDS
        
;;
;;CAUSES MODEM TO DISCONNECT AND GO ON-HOOK
;;
SHOFF1: TM      IMAGE35,#CARRIER        ;;SEE IF CARRIER IS PRESENT
        JR      NZ,SHOFF                ;;NO SEND SPACE IF LOST CARRIER
        TM      OFLAGS,#LSD             ;;LONG SPACE DISCONNECT ENABLED?
        JR      Z,SHOFF                 ;;JUMP IF NOT
        OR      PFLAGS3,#SBREAK         ;;ELSE SEND BREAK FOR 4 SECONDS
        LD      R4,#4
        CALL    WAITSEC
SHOFF:  CALL    CD$OFF                  ;;TURN OFF CARRIER DETECT LED
        LD      IMAGE34,#ALB            ;;RING DETECT MODE FOR RECEIVER 
        CALL    SETVOL                  ;;SPEAKER OFF
        AND     IMAGE35,#"(SWHOOK+SHUNT+RJ13)   ;;OPEN SWITCH HOOK RELAYS
        OR      IMAGE35,#RING+CARRIER   ;;MAKE RS232 RI AND CD FALSE    
        AND     PFLAGS3,#"(TXON+MOD$DIS+SUSM+SBREAK)  ;;KILL MODEM TRANSMITTER
        AND     PFLAGS,#"(CONEST+ONLINE)    ;;KILL CONNECTION ESTABLISHED FLAG
        CLR     LPBACK                  ;;CLEAR ALL TEST MODES
        CALL    UD3435                  ;;UPDATE 34,35
        EI                              ;;ENABLE INTERRUPTS
        JP      WAIT250                 ;;WAIT 250 MS FOR TELCO TO DISCONNECT
;;
;;      
;;
DIALER$TB:
        SRP     INTREG
        LD      R0,#SINE2DB > 8 ;;POINT TO 2 DB LOW SINE TABLE
        LD      R1,R14
        LDE     R2,@RR0         ;;GET THE SINE OF LOW TONE VECTOR INTO R2
        LD      R0,#SINE256 > 8 ;;POINT TO NORMAL SINE TABLE FOR HIGH TONE
        LD      R1,R12          ;;
        LDE     R0,@RR0         ;;GET SINE OF HIGH TONE VECTOR INTO R0
        ADD     R2,R0           ;;SUM THE VECTORS
        RRC     R2              ;;DIVIDE BY 2
        LD      DAC,R2          ;;OUT TO THE DAC
        CLR     R2              ;;R2 MUST BE ZERO SO NOT TO SCREW UP
;;                              ;;/  THE MAIN TIME BASE ROUTINE!
        ADD     R13,PRIMREG+3   ;;ADVANCE PHASOR 1
        ADC     R12,PRIMREG+2
        ADD     R15,R11         ;;ADVANCE PHASOR 2
        ADC     R14,R10
        SRP     PRIMREG         ;;BACK TO PRIMARY REGISTERS
        DEC     R0              ;;DEC R0 AS A PRESCALER
        TM      IRQ,#8          ;;TEST FOR KEY PRESSED
        JR      NZ,ABORT$DIAL   ;;QUIT IF TRUE
        TM      PORT3,#DTR      ;;TEST FOR FALSE DTR LINE
        JR      Z,ABORT$DIAL    ;;QUIT IF FALSE
TB$DONE:IRET
;;
;;
ABORT$DIAL:
        LD      RESULTS,#'0'            ;;OK RESULT CODE
ABORT:  DI                              ;;NO INTERRUPTS NOW
        LD      T1,#MC9600              ;;9600 HZ MASTER CLOCK
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET VECTOR TO MAIN HS TIMEBASE
        LD      IRQ5VL,#TIMEBASE & 255  ;;  /
        CALL    ZERO$FREQ               ;;KILL TRANSMITTER
        CALL    SHOFF                   ;;HANG UP PHONE
        JP      GETCOMM
;;
;;

;;---------------------------------------------------------------
;;
;;THIS PUTS THE MODEM IN ANSWER MODE FOR INCOMING CALLS
;;
ANSWR:  LD      BITCOUNT,CDW            ;;GET "WAIT FOR CARRIER" TIME 
        TM      OFLAGS,#BELLOPT         ;;SEE IF BELL MODE
        JR      Z,CCITTANS              ;;JUMP IF CCITT V.22 (1200 ONLY)
        TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;TEST FOR HIGH SPEED
        JR      NZ,ANSWR1               ;;JUMP IF LOW
        TM      LPBACK,#1               ;;TEST FOR LOOP BACK MODE
        JR      NZ,ANSW$LPB             ;;JUMP IF HIGH SPEED LOOP BACK
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,ANSW12               ;;/ YES, GO TO HIGH SPEED ANSWER NOW
ANSWR1: CALL    LSTXINIT                ;;SET UP TX FOR LOW SPEED OPERATION
        OR      PFLAGS3,TXFLAG          ;;TURN ON TRANSMITTER
ANSWR3: CALL    LSRXIA                  ;;SET UP LOW SPEED RECEIVER
        CALL    WAIT$FSK$CD             ;;WAIT FOR A SIGNAL
        JP      NC,NO$CARRIER           ;;ABORT IF NO CARRIER
        LD      R0,#40                  ;;SET FOR 200 MS MARK QUALIFICATION
        CALL    FSKM                    ;;SEE IF ITS A MARK FOR 200 MS
        JR      NC,ANSW12               ;;JUMP IF NOT A MARK (POSSIBLE 1200)
ANSWR2: JP      ORIG$LOW2
;;
;;THIS IS THE CCITT V.22 ANSWER HANDSHAKE SEQUENCE
;;
CCITTANS:
        CALL    HSTXON                  ;;TURN ON HIGH SPEED TRANSMITTER
        OR      PFLAGS3,#MOD$DIS        ;;DISABLE MODULATION
        LD      INTREG+10,#F2100 > 8    ;;CHANGE CARRIER TO 2100 HZ
        LD      IMAGE34,#PSK+BELL       ;; // (ECHO SUPPRESSOR TONE)
        CALL    UPDATE34                ;;MAKE SLAVE Z8 SET FILTERS TO ANSWER
        LD      R4,#3
        CALL    WAITSEC                 ;;WAIT 3 SECONDS
        AND     PFLAGS3,#"TXON          ;;TURN OFF ECHO SUPPRESS TONE
        LD      CT1200,#90              ;;SET FOR 75 MS DELAY
DLY75:  OR      CT1200,CT1200
        JR      NZ,DLY75                ;;WAIT HERE FOR 75 MS
        CALL    HSTXON                  ;;ENABLE 2400 HZ PSK TRANSMITTER
        OR      PFLAGS3,#SUSM           ;;SEND UNSCRAMBLED MARKS
        CALL    HSRXIA                  ;;ENABLE PSK RECEIVER
        CALL    WAIT$PSK$CD             ;;WAIT FOR CCITT PSK CARRIER
        JP      NC,NO$CARRIER
        AND     PFLAGS3,#"SUSM          ;;ENABLE SCRAMBLER
        JR      ANSW13                  ;;GO INTO DATA MODE
;;
;;
ANSW$LPB:
        CALL    HSTXON                  ;;IF LOOPBACK TURN ON XMITTER NOW
ANSW12: CALL    HSRXIA                  ;;INITIALIZE HIGH SPEED RECEIVER
        SUB     BITCOUNT,#2             ;;COUNT EXTRA 2 SECONDS
        JP      ULE,NO$CARRIER          ;;GIVE UP AFTER TIMEOUT (ZERO OR CARRY)
        PUSH    BITCOUNT                ;;SAVE SECONDS TIMEOUT COUNTER
        LD      BITCOUNT,#2             ;;SET TO LOOK FOR PSK MARK FOR 2 SEC.
        CALL    WAIT$PSK$CD             ;;WAIT FOR HIGH SPEED CARRIER
        POP     BITCOUNT
        JP      NC,ANSWR3               ;;IF NO CARRIER TRY LOW SPEED AGAIN
        CALL    HSTXON                  ;;SET TRANSMITTER FOR HIGH SPEED
ANSW13: CALL    WAIT250                 ;;WAIT 1/2 SECOND
        CALL    WAIT250                 ;;
        JR      AWAIT17                 ;;THEN GO ON-LINE
;;
        
;;------------------------------------------------------------
;;
;;THIS PUTS THE MODEM IN LOW SPEED ORIGINATE MODE FOR OUT-GOING CALLS
;;
ORIG$LOW:
        CALL    LSTXINIT                ;;INITIALIZE LOW SPEED TRANSMITTER
        CALL    LSRXIO                  ;;INIT. LOW SPEED RECEIVER
        TM      LPBACK,#1               ;;CHECK FOR ANALOG LOOP BACK
        JR      Z,OL$NORM               ;;JUMP IF NORMAL
        OR      PFLAGS3,TXFLAG          ;; /ELSE TURN ON TRANSMITTER NOW
OL$NORM: LD     BITCOUNT,CDW            ;;GET CARRIER DETECT WAIT TIME
        CALL    WAIT$FSK$CD             ;;WAIT FOR LOW SPEED CARRIER
        JP      NC,NO$CARRIER
        CALL    FSK$MARK
        JP      NC,NO$CARRIER
        OR      PFLAGS3,TXFLAG  
ORIG$LOW2:
        OR      IMAGE35,#HS$LED         ;;TURN OFF HS LED
        LD      RESULTS,#'1'            ;;LOW SPEED CONNECT RESULT CODE
        JR      LSENTRY                 ;;NOW GO DO LOW SPEED MODEM ONLINE
;;
;;---THIS PUTS THE MODEM IN HIGH SPEED ORIGINATE MODE--------   
;;
ORIG$HIGH:
        LD      BITCOUNT,CDW            ;;GET CARRIER DETECT WAIT TIME
        TM      LPBACK,#1               ;;TEST FOR LOOP BACK MODE
        JR      NZ,OH001                ;;JUMP IF LOOP BACK
        CALL    LSRXIO                  ;;INIT. LOW SPEED  RECEIVER
        CALL    WAIT$FSK$CD             ;;WAIT FOR LOW SPEED CARRIER
        JP      NC,NO$CARRIER
OH001:  CALL    HSRXIO                  ;;INITIALIZE HIGH SPEED RECEIVER
        CALL    HSTXON                  ;;TURN ON HIGH SPEED TRANSMITTER
        CALL    WAIT$PSK$CD             ;;WAIT FOR HIGH SPEED CARRIER
        JP      NC,NO$CARRIER
AWAIT17:AND     IMAGE35,#"HS$LED        ;;TURN ON HIGH SPEED LED
        LD      R1,#4
        CLR     CT1200
RDLCK:  TM      PFLAGS3,#SUSM           ;;SENDING DIGITAL LOOP REQUEST?
        JR      Z,CONN12                ;;JUMP IF NOT
        CALL    WAIT250                 ;;WAIT 250 MS
        DEC     CT1200                  ;;CT1200=255 IF RDL REQUEST ACTIVE
        DJNZ    R1,RDLCK                ;;GIVE OTHER MODEM 1 SEC. TO RESPOND
        AND     PFLAGS3,#"SUSM          ;;QUIT SENDING SCRAMBLED MARK
        LD      RESULTS,#'4'            ;;ERROR RESULT
        JP      GETCOMM                 ;; / BACK TO COMMAND MODE
CONN12: CALL    WAIT                    ;;WAIT 0 OR 212 MS IF RDL REQUEST
        LD      RESULTS,#'5'            ;;CONNECT 1200 RESULT
        DI
        LD      IMR,#20H                ;;TURN OFF ALL INT. EXCEPT TRANSMIT
        EI
LSENTRY:CALL    RESULT                  ;;SEND RESULT CODE
        CALL    CD$ON                   ;;TURN ON CARRIER DETECT LED
        CALL    SPKCRL                  ;;UPDATE SPEAKER
        TM      IMAGE34,#PSK            ;;TEST FOR HIGH SPEED MODE
        JR      Z,CLOOPB                ;;JUMP IF LOW SPEED
        DI
        LD      R7,#0E0H                ;;MUTE RECEIVER
        LD      IMR,#34H                ;;TURN ON RECEIVER AGAIN
        AND     IRQ,#20H                ;;KILL PENDING INTERRUPTS
        AND     PFLAGS2,#"TCA           ;;KILL TX CHAR AVALABLE FLAG
        LD      CT1200,#60              ;;SET UP 50 MS DELAY
        EI                              ;;DELAY FOR NOISE TO GO AWAY
CLOOPA: CP      CT1200,#0               ;;  / 50 MS DELAY
        JR      NZ,CLOOPA
        LD      R7,#0E4H                ;;UNMUTE RECEIVER
CLOOPB: TM      IMAGE34,#PSK            ;;SEE IF HIGH OR LOW SPEED MODE
        JR      Z,CL00A                 ;;JUMP IF LOW SPEED
        OR      PFLAGS,#(OBAUD0+OBAUD1+IBAUD0+IBAUD1)  ;;SET FOR FAST RESULTS
        JR      CL00B
CL00A:  TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;HIGH SPEED LAST TIME?
        JR      NZ,CL00B                ;;NO, LEAVE BAUD BITS ALONE
        AND     PFLAGS,#"(OBAUD0+IBAUD0);;YES, NOW SET FOR 300 BAUD RESULTS
        OR      PFLAGS,#(OBAUD1+IBAUD1)
CL00B:  CLR     R1                      ;;CLEAR ESCAPE TIMER
        CLR     BITCOUNT                ;;CLEAR ESCAPE CHARACTER COUNTER
        OR      PFLAGS,#ONLINE+CONEST   ;;SET ON-LINE STATUS
        TM      OFLAGS,#LSD             ;;LONG SPACE DISCONNECT ENABLED?
        JR      Z,CLOOP                 ;;JUMP IF NOT
        OR      IMAGE34,#SQRQ           ;;SET LONG SPACE DISC. BIT
        CALL    UD34XX                  ;;UPDATE34 WITHOUT CON2 STROBE
CLOOP:
        CP      CT1200,#0               ;;SERVICE GUARD TIME COUNTERS
        JR      NE,CLOOP1               ;;  /
        LD      CT1200,#24              ;;20 MS TIMEOUT
        CALL    DTRCHECK                ;;TEST DTR AND ABORT IF FALSE   
        CALL    CD$UPDATE               ;;MAKE CD LED AGREE WITH CARRIER STATUS
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,CL000
        TM      PFLAGS2,#CDLED          ;;SEE IF CARRIER IS PRESENT
        JR      Z,CD$RECOVER            ;; / (UPDATED BY CD$UPDATE)
CL000:  OR      R1,R1                   ;;TEST GUARD TIME COUNTER FOR ZERO
        JR      Z,CLOOP1                ;; /
        DEC     R1                      ;;DECREMENT IF IT WASNT ZERO
CLOOP1: TM      PFLAGS2,#TCA            ;;TEST FOR TRANSMITTER CHAR AVALABLE
        JR      Z,CL005                 ;;JUMP IF ZERO (NO CHARACTER)
        AND     PFLAGS2,#"TCA           ;;CLEAR THE TX CHAR FLAG
        TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;TEST FOR HIGH SPEED
        JR      NZ,CL001                ;;JUMP IF LOW SPEED
        TM      PFLAGS,#DPLX            ;;TEST FOR HALF DUPLEX MODE
        JR      NZ,CL001                ;;JUMP IF FULL DUPLEX (NO ECHO)
        LD      R10,INTREG+11           ;;PUT TRANSMITTED CHAR IN RX HOLDING
        OR      R7,#10H                 ;;SET DATA RECEIVED FLAG TRUE
CL001:  AND     INTREG+11,#7FH          ;;CLEAR PARITY BIT ON TX CHAR
        TM      OFLAGS,#STUPID          ;;ESCAPE SEQUENCE ALLOWED?
        JR      NZ,CLOOP                ;; / JUMP IF ESCAPE NOT ALLOWED
;;
;;ESCAPE CODE DETECTOR
;;
        CP      BITCOUNT,#0             ;;IS THIS THE FIRST ESCAPE CHARACTER?
        JR      NZ,CL002                ;;JUMP IF NOT FIRST CH AFTER GUARD TIME
        OR      R1,R1                   ;;TEST FOR EXPIRED GUARD TIME
        JR      NZ,CL003                ;;/ JUMP IF NOT EXPIRED
CL002:  CP      INTREG+11,ESCODE        ;;IS THIS AN ESCAPE CHARACTER?
        JR      NE,CL003                ;;/ JUMP IF NOT
        INC     BITCOUNT                ;;COUNT THIS ESCAPE CHARACTER
        JR      CL004                   ;;
;;
CL003:  CLR     BITCOUNT                ;;CLEAR ESCAPE CHARACTER COUNTER
CL004:  LD      R1,EGT                  ;;RELOAD GUARD TIME COUNTER
CL005:  OR      R1,R1                   ;;IS GUARD TIME EXPIRED?
        JR      NZ,CLOOP                ;; / JUMP IF NOT
        CP      BITCOUNT,#3             ;;3 ESCAPE CHARS YET?
        JR      EQ,ESC$DET              ;; / JUMP IF 3 HAVE BEEN RECEIVED
        CP      EGT,#0                  ;;IS GUARD TIME SET TO ZERO?
        JR      Z,CLOOP                 ;;/ YES, LOOP BACK TO TOP
        CLR     BITCOUNT                ;;ELSE CLEAR ESCAPE CHARACTER COUNTER
        JR      CLOOP                   ;; / THEN GO BACK TO TOP
ESC$DET: LD     R7,#0E0H                ;;MUTE RECEIVER
        JP      OK                      ;;GOTO COMMAND PROCESSOR
;;---------
CD$RECOVER:                             ;;COME HERE WHEN CARRIER IS LOST
        LD      R2,#255
        LD      R1,#255
RECOV1: INC     R1                      ;;ADVANCE LOST CARRIER DISCONN. TIMER
        INC     R2                      ;; / AND CARRIER DELAY TIMER
        LD      CT1200,#120             ;;SET FOR 100 MS DELAY
RECOV2: CP      CT1200,#0               ;;100 MS UP?
        JR      EQ,RECOV1               ;;YES, ADVANCE TIMERS
        TM      IMAGE35,#CARRIER        ;;/NO, TEST FOR PRESENCE OF CARRIER
        JR      Z,RECOV3                ;;JUMP IF CARRIER PRESENT
        CLR     R2                      ;;/  ELSE JUST RESET CARRIER TIMER
        AND     LPBACK,#"S16RDL         ;;/AND CLEAR REMOTE DIGITAL LOOP BACK
        JR      RECOV4                  ;;
RECOV3: CP      R2,CDELAY               ;;CARRIER PRESENT LONG ENOUGH?
        JP      UGE,CL00B               ;; / YES, GO BACK ON LINE
RECOV4: CP      R1,LCDT                 ;;CARRIER GONE LONG ENOUGH?
        JP      EQ,NO$CARRIER           ;; / YES, EXIT
        JR      RECOV2                  ;;/  ELSE WAIT SOME MORE
;;
;;---------
CD$UPDATE:
        TM      IMAGE35,#CARRIER        ;;CHECK STATE OF HARDWARE CARRIER DET.
        JR      Z,CD$ON1                ;; / JUMP IF IT IS ON
        TM      PFLAGS2,#CDLED
        JR      Z,CD$RET
CD$OFF: PUSH    IMR
        DI
        LD      R7,#0E0H                ;;   MUTE RECEIVER      
        LD      PORT3,R7                ;;   CLAMP RS232 DATA TO MARK
        LD      R0,#0E0H                ;; / SET TO TURN OFF CD LED
        AND     PFLAGS2,#"CDLED         ;;/  SAVE STATE OF CD-LED
        JR      SETRI2                  ;;
;;
CD$ON1: TM      PFLAGS2,#CDLED          ;;SEE IF LED WAS ON ALREADY
        JR      NZ,CD$RET               ;;DONT BOTHER IF LED ALREADY ON
CD$ON:  PUSH    IMR
        DI
        OR      PFLAGS2,#CDLED          ;;SAVE NEW STATE OF CD-LED
        LD      R0,#60H                 ;;CD LED ON
        LD      R7,#0E4H                ;;UN-MUTE RECEIVER
SETRI2: LD      PORT0,R0                ;;ADDRESS THE BIT IN THE LATCH
        NOP                             ;;DELAY
        AND     PORT3,#"20H             ;;STROBE THE ADDRESSABLE LATCH P35
        OR      PORT3,#20H
        POP     IMR
CD$RET: RET

;;-------------------------------------------------------------
;;INITIALIZE THE HIGH SPEED RECEIVER HARDWARE AND INTERRUPT DRIVERS.
;;
;;COME HERE FOR ORIGINATE 1200 
;;
HSRXIO: LD      IMAGE34,#(PSK+ORIG+BELL)  ;;ORIGINATE MODE INIT.
        JR      HSRXI
;;
;;COME HERE FOR ANSWER 1200
;;
HSRXIA: LD      IMAGE34,#(PSK+BELL)     ;;ANSWER MODE INIT.
        TM      OFLAGS,#BELLOPT         ;;1=BELL  0=CCITT V.22
        JR      NZ,HSRXI                ;;JUMP IF BELL
        AND     IMAGE34,#"BELL          ;;ELSE CLEAR BELL-212 BIT
;;
HSRXI:  DI
        LD      IRQ4PTR,#2              ;;POINT IRQ4 TO 1219 HZ INT. ROUTINE
        LD      PRE0,#0DH               ;;DIVIDE BY N=3 FROM 921.6 KHZ
        CLR     T0                      ;;307.2KHZ DIV. BY 256 FOR 1200 HZ
        OR      TMR,#3                  ;;START TIMER 0 RUNNING
        LD      IMR,#34H                ;;ENABLE RECEIVER INTERRUPTS
HSRXI2: LD      R7,#0E0H                ;;INIT. RECEIVER FLAGS
        EI
        TM      LPBACK,#1               ;;TEST FOR LOOPBACK
        JR      Z,HSRXI3                ;;JUMP IF NO LOOPBACK
        OR      IMAGE34,#ALB            ;;ELSE SET LOOPBACK BIT
HSRXI3: CALL    UPDATE34                ;;SET HARDWARE
        RET
;;
;;------------------------------------------------------------------
;;INITIALIZE THE LOW SPEED RECEIVER HARDWARE AND INTERRUPT DRIVERS
;;
;;COME HERE FOR ORIGINATE 300 MODE.
;;
LSRXIO: LD      IMAGE34,#(ORIG+BELL)    ;;LOW SPEED ORIGINATE
        JR      LSRXI
;;
;;COME HERE FOR ANSWER 300 MODE.
;;
LSRXIA: LD      IMAGE34,#BELL           ;;BELL 103 OPERATION
;;
LSRXI:  DI
        LD      IMR,#20H                ;;ONLY MAIN TIMEBASE INT. ACTIVE
        JR      HSRXI2                  ;;GO FINISH INIT. ELSEWHERE
;;
;;---------------------------------------------------------------
;;
;;
;;----------------------------------------------------------
;;THIS SETS THE HIGH SPEED MODEM TRANSMITTER TO THE CORRECT
;;FREQUENCY BASED ON THE MODE CONTROL BIT.
;;"HSTXINIT" DOES NOT TURN ON THE TRANSMITTER BUT "HSTXON" DOES.
;;** R0 IS MODIFIED
;;
HSTXON: OR      PFLAGS3,TXFLAG          ;;COME HERE TO TURN ON TRANSMITTER
HSTXINIT:
        AND     PFLAGS3,#"(SUSM+MOD$DIS) ;;ENABLE SCRAMBLER AND MODULATION
        DI
        LD      T1,#MC9600              ;;9600 HZ MASTER CLOCK
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET VECTOR TO HS TRANSMITTER
        LD      IRQ5VL,#TIMEBASE & 255  ;; /
        CALL    ZF0                     ;;CLEAR TRANSMITTER VARIABLES
        LD      INTREG+10,#F1200 > 8    ;;ASSUME ORIGINATE CARRIER 1200 HZ
        LD      INTREG+13,#SINE256 > 8  ;; SINETABLE POINTER
        TM      PFLAGS,#MODE            ;;TEST FOR ORIG. / ANSW. MODES
        JR      NZ,HSTX1                ;;JUMP IF ORIGINATE
        RL      INTREG+10               ;;ORG CARRIER X 2 = 2400 HZ
HSTX1:  LD      COUNTER,#1              ;;INIT ASYNC. TO SYNC CONVERTER
        LD      TXDATA,#255             ;; /
        EI                              ;; /
        RET
;;
;;----------------------------------------------------------------
;;INITIALIZE THE LOW SPEED TRANSMITTER
;;NOTE: IT IS NOT TURNED ON WITH THIS ROUTINE
;;
LSTXINIT:
        DI
        LD      T1,#MC7200              ;;7200 HZ MASTER CLOCK
        LD      IRQ5VH,#LOW$TIMEBASE >8 ;;POINT JUMP VECTOR TO LOW SPEED 
        LD      IRQ5VL,#LOW$TIMEBASE & 255
        SRP     INTREG
        LD      R6,#F2025 > 8           ;;ASSUME ANSWER MODE FIRST
        LD      R7,#F2025 & 255
        LD      R8,#F2225 > 8
        LD      R9,#F2225 & 255
        TM      PFLAGS,#MODE            ;;TEST FOR ORIG OR ANSW
        JR      Z,LSDONE                ;;JUMP IF ANSWER
        LD      R6,#F1070 > 8           ;;ELSE SET UP ORIGINATE FREQS.
        LD      R7,#F1070 & 255
        LD      R8,#F1270 > 8
        LD      R9,#F1270 & 255
LSDONE: CLR     R12
        CLR     R13                     ;;CLEAR CARRIER REGISTERS
        LD      R11,#255
        CLR     COUNTER
        LD      R15,#3                  ;;ASSUME 300 BAUD
        TM      PFLAGS,#OBAUD1
        JR      NZ,LS1
        LD      R15,#8                  ;;OR SET FOR 110 BAUD
LS1:    LD      R14,R15
        LD      R5,#1
        SRP     PRIMREG                 ;;RESTORE PRIMARY REGISTER SET
        EI
        RET
;;
;;-----------------------------------------------------------------
;;HIGH SPEED MODE RECEIVER CARRIER DETECT.
;;RETURNS CARRY SET IF CARRIER DETECTED
;;
WAIT$PSK$CD:
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,AWC11A               ;; / YES, RETURN AS IF CARRIER PRESENT
        CLR     R3                      ;;CLEAR MARK COUNTER
AWC9:   LD      R0,#10                  ;;COUNT 100 MS INTERVALS
AWC10:  LD      CT1200,#120             ;;SET FOR 100 MS TIMEOUT
AWC11:  CP      R3,#120                 ;;120 MARK BITS? (COUNTED BY "DCLOCK")
        JR      ULE,AWC12               ;;120 MARKS=100 MILLISECONDS DELAY
AWC11A: SCF                             ;;SET CARRY FLAG
        AND     IMAGE35,#"HS$LED        ;;ENABLE HS LED
AWC11B: RET
;;
AWC12:  CALL    KP$ABORT                ;;ABORT IF KEY PRESSED
        CP      CT1200,#0               ;;TEST FOR 100 MS TIMEOUT
        JR      NE,AWC11
        CALL    DTRCHECK                ;;CHECK ON FALSE DTR
        DJNZ    R0,AWC10                ;;JUMP IF 1 SECOND NOT UP
        DEC     BITCOUNT                ;;COUNT DOWN MAX SECONDS
        JR      NZ,AWC9
AWC13:  RCF
        RET                             ;;RETURN CARRY CLEAR IF TIME UP
;;-------------------------------------------------------------------
;;WAITS FOR STEADY MARK CARRIER TONE AND RETURNS CARRY SET IF CARRIER PRESENT.
;;RETURNS CARRY CLEAR IF: KEY PRESSED, DTR FALSE,
;; OR NO CARRIER WITHIN CARRIER DETECT WAIT TIME SET BY S7  "CDW".
;;MODIFIES R0,R1,R2,R3,R4,BITCOUNT,CT1200,PORT0
;;
WAIT$FSK$CD:
        EI
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,WFC4B                ;;/ EXIT AS IF CARRIER PRESENT IF EQ
        CLR     R0                      ;;R0, R1 ARE CARRIER PRESENT TIMERS
        CLR     R1
WFC1:   LD      R2,#100                 ;;R2 COUNTS SECONDS
WFC2:   LD      CT1200,#12              ;;10 MS DELAY
WFC3:   CP      CT1200,#0
        JR      NE,WFC3 
        TM      PORT3,#STATUS           ;;TEST FOR CARRIER ENERGY
        JR      Z,WFC5                  ;;JUMP IF NO CARRIER PRESENT
        TM      PFLAGS,#MODE            ;;TEST FOR ORIG/ANSW
        JR      Z,WFC4                  ;;DONT CARE ABOUT MARK IF ANSWER MODE
        TM      PORT2,#RDATA            ;;TEST FOR MARK OR SPACE
        JR      Z,WFC5                  ;;JUMP IF NOT MARK
WFC4:   INC     R1
        CP      R1,#10                  ;;100 MS OF CARRIER YET?
        JR      NE,WFC4A                ;;JUMP IF NOT
        INC     R0
        CLR     R1                      ;;ELSE COUNT 1/10 SECONDS
WFC4A:  CP      R0,CDELAY               ;;CARRIER DELAY EXPIRED?
        JR      NE,WFC6                 ;;JUMP IF CARRIER NOT PRESENT
WFC4B:  SCF                             ;;SET CARRY
        RET                             ;;AND RETURN WITH CARRIER PRESENT
;;
WFC5:   CLR     R0
        CLR     R1
WFC6:   CALL    KP$ABORT                ;;CHECK FOR KEY PRESSD
        DJNZ    R2,WFC2                 ;;COUNT 1/10 SECOND
        CALL    DTRCHECK                ;;CHECK FOR FALSE DTR
        DEC     BITCOUNT                ;;COUNT SECONDS         
        JR      NZ,WFC1
WFC7:   RCF                             ;;RESET CARRY FLAG
        RET                             ;;RETURN WITH NO CARRIER
;;
DTRCHECK:
        TM      PORT3,#DTR
        JR      Z,NO$CARRIER
        RET
;;
KP$ABORT:
        TM      OFLAGS,#STUPID          ;;DUMB MODE?
        JR      NZ,KPA1                 ;;DONT LOOK AT KEYBOARD IF DUMB
        TM      IRQ,#8                  ;;KEY PRESSED?
KPA0:   JR      NZ,NO$CARRIER           ;;ABORT IF TRUE
KPA1:   RET
;;
NO$CARRIER:
        LD      RESULTS,#'3'    ;;NO CARRIER RESULT
        JP      ABORT           ;;EXIT TO COMMAND LEVEL VIA ABORT ROUTINE
;;
;;
;;---------------------------------------------------
;;LOOKS FOR AN FSK MARK TONE FOR 100 MS.
;;IF MARK IS PRESENT FOR THE ENTIRE PERIOD IT RETURNS CARRY SET.
;;IF SPACE IS DETECTED IT RETURNS WITH CARRY CLEAR.
;;DISTURBS  R0,R1,PORT0,CT1200
;;
FSK$MARK:
        LD      R0,#20          ;;FOR 100 MS TIMEOUT
FSKM:   EI
        CP      LCDT,#255       ;;IGNORE CARRIER?
        JR      EQ,FSKM2A       ;;YES, RETURNS AS IF CARRIER PRESENT
FSKM0:  LD      CT1200,#6       ;;SET FOR 5 MS TIMEOUT
FSKM1:  CP      CT1200,#0
        JR      NZ,FSKM1        ;;LOOP HERE FOR 5 MS
        TM      PORT2,#RDATA    ;;TEST RX DATA BIT
        JR      Z,FSKM3         ;;JUMP IF NOT A MARK
        DJNZ    R0,FSKM0        ;;LOOP 20 TIMES
FSKM2A: SCF                     ;;SET CARRY
        RET                     ;;MARK PRESENT
FSKM3:  RCF                     ;;ELSE CLEAR CARRY
        RET                     ;;MARK NOT PRESENT
;;
;;--------THIS IS THE C0,C1 COMMAND--------------------------
;;
SETCARRIER:
        CALL    NEXTNUM
        JR      NZ,CXON
        AND     PFLAGS3,#"TXON          ;;CARRIER OFF
        CLR     TXFLAG                  ;;NEVER TO COME ON AGAIN!
        RET
;;
CXON:   DI              
        TCM     PFLAGS,#(OBAUD0+OBAUD1)  ;;TEST FOR HIGH/LOW SPEED
        JR      Z,CXON1                 ;; / JUMP IF HIGH SPEED
        CALL    LSTXINIT                ;;SET UP LOW SPEED XMITTER
        JR      CXON2
CXON1:  CALL    HSTXINIT                ;;SET UP HIGH SPEED TRANSMITTER
CXON2:  LD      TXFLAG,#TXON            ;;SET TRANSMITTER CONTROL FLAG
        OR      PFLAGS3,TXFLAG          ;;TURN ON TRANSMITTER
        RET             ;;NOTE: INTERRUPTS ENABLED BY "LSTXINIT" OR "HSTXINIT".
;;
;;---------------------------------------------------------------
;;THESE ARE THE TIME DELAY ROUTINES
;;
;;
WAIT2SEC:
        LD      R4,PT           ;;GET PAUSE TIME FOR COMMA
WAITSEC:AND     R4,R4           ;;TEST R4 FOR ZERO
        JR      Z,WAIT$DONE     ;;EXIT WHEN R4 IS ZERO
        CALL    WAIT250
        CALL    WAIT250
        CALL    WAIT250
        CALL    WAIT250         ;;WAIT 1 SECOND
        DEC     R4
        JR      WAITSEC
;;
WAIT250:
        LD      CT1200,#150     ;;WAITS 250 MS
        CALL    WAIT
W120:   LD      CT1200,#150     ;;WAITS 125 MS
WAIT:   OR      CT1200,CT1200   ;;TEST FOR ZERO
        JR      NZ,WAIT         ;;LOOP TIL CT1200 IS ZERO
WAIT$DONE:
        TM      OFLAGS,#VBON    ;;RETURN STATE OF VERBOSE FLAG
        RET


;;-------------------------------------------------------
;;RECEIVER SYNC TO ASYNC CONVERTER
;;
;;RR14 USED AS DIBIT AND DATA BIT BUFFER ( GLOBAL!)
;;CONTENTS OF RR12 DESTROYED
;;
;;REGISTERS USED:
;;      R15     SERIAL STATUS LINE SHIFT REGISTER  
;;              LOW 6 BITS ARE SIGNAL QUALITY
;;              00XX XXXX = NO CARRIER
;;              11XX XXXX = NORMAL CARRIER DETECT
;;              01XX XXXX = REMOTE DIGITAL LOOPBACK ACKNOWLEDGE ALT. 1s and 0s
;;              10XX XXXX = REMOTE DIGITAL LOOPBACK REQUEST  UNSCRAMBLED MARK
;;      R14     BIT 7 HOLDS DESCRAMBLED DATA BIT
;;      R13     BIT COUNTER FOR SERIAL STATUS LINE
;;      R12     SCRATCH
;;      R11     LINE RATE SHIFT REGISTER
;;      R10     RECEIVER HOLDING REGISTER
;;      R9      OUTPUT SHIFT REGISTER
;;      R8      TIME DELAY COUNTER
;;      R7      STATUS FLAGS
;;      R6      OUTPUT BIT COUNTER
;;      R5      INPUT BIT COUNTER
;;      R4      COUNTS CONTINUOUS ZERO BITS (FOR BREAK DETECTION)
;;      R3      COUNTS 1 BITS
;;R7 BIT MAP:
;;      BIT 2   1=OK TO SEND RX DATA TO RS232    0= RX DATA MUTED
;;      BIT 3   NOT USED
;;      BIT 4   CHARACTER IN RECEIVER HOLDING REGISTER IS READY
;;      BIT 5   ALWAYS 1
;;      BIT 6   ALWAYS 1
;;      BIT 7   NEXT BIT TO BE OUTPUT TO RS232
;;
        BRKLEN: EQU     20              ;;NUMBER OF ZEROS BITS IN BREAK
        DCD:    EQU     11000000B       ;;R15 CARRIER DETECT MASK
        RDLREQ: EQU     01000000B       ;;REMOTE DIGITAL LOOP REQUEST MASK
        RDLACK: EQU     10000000B       ;;REMOTE DIGITAL LOOP ACKNOWLEDGE
;;
;;-------------------------------------------------------------
;;
DCLOCK:                         ;;IRQ2 DESTINATION FOR PSK MODE
        LD      R14,PORT2
        AND     R14,#RDATA      ;;KEEP ONLY DATA BIT (BIT 0)
        TM      PORT3,#STATUS   ;;TEST STATUS LINE
        JR      Z,DB9
        OR      R14,#2          ;;PUT STATUS BIT INTO BIT 1
DB9:    EI
        RR      R14             ;;GET RX DATA IN BIT 7 OF R14 AND CARRY FLAG
        INC     R3              ;;ASSUME MARK BY ADVANCING MARK COUNTER
        JR      C,DB10          ;;JUMP IF A MARK
        CLR     R3              ;;ELSE CLEAR THE MARK COUNTER
DB10:   TM      R8,#255         ;;TEST DELAY TIMER
        JR      MI,DB11         ;;JUMP IF MINUS
        DEC     R8              ;;ELSE COUNT DOWN
DB11:   JR      NZ,DB12         ;;JUMP IF NOT ZERO YET
        OR      R7,#10H         ;;IF ZERO SET THEN SET READY FLAG
DB12:   OR      R5,R5           ;;TEST R5 (INPUT BIT COUNTER)
        JR      NZ,DB14         ;;JUMP IF IN A CHARACTER
        TM      R14,#80H        ;;TEST FOR START BIT
        JR      NZ,DB15         ;;JUMP IF NO START BIT
        CP      R4,#BRKLEN      ;;BREAK DETECTED?
        JR      UGE,INSTOP      ;;/ YES, THEN JUMP AS IF STOP BIT FOUND
        LD      R5,#1           ;;START BIT FOUND, SET BIT COUNTER=1
        LD      R4,#1           ;;AND SET SPACE COUNTER=1
        JR      DB13            ;;EXIT
;;
;;
DB14:   CP      R5,#9           ;;SEE IF WE ARE IN THE STOP BIT
        JR      UGE,INSTOP
        INC     R5              ;;COUNT THIS DATA BIT
        LD      FLAGS,R14       ;;PUT RECEIVED DATA BIT INTO CARRY FLAG
        RRC     R11             ;;THEN SHIFT CARRY INTO LINE SHIFT REGISTER
        JR      MI,DB15         ;;JUMP IF BIT WAS A 1
        INC     R4              ;;ELSE ADVANCE SPACE COUNTER IF ZERO
        JR      DB16
DB15:   CLR     R4              ;;CLEAR COUNTER IF BIT WAS 1 (CLEAR BREAK)
DB16:   CP      R4,#BRKLEN      ;;TEST FOR 20 SPACE BITS (BREAK)
        JR      LT,DB13         ;;JUMP IF NOT BREAK
;;
INSTOP: CLR     R5              ;;RESET INPUT BIT COUNTER
        TM      R14,#80H        ;;TEST FOR MISSING STOP BIT
        JR      NZ,GOTSTOP      ;;JUMP IF ITS THERE
        CP      R4,#BRKLEN      ;;TEST FOR BREAK
        JR      UGE,BREAK$X     ;;DONT COUNT START BIT IF BREAK
        INC     R5              ;;  / ELSE COUNT THE START BIT
        INC     R4              ;;   / AND BUMP SPACE COUNTER
GOTSTOP:LD      R10,R11         ;;PUT DATA BYTE INTO HOLDING REGISTER
        LD      R8,#4           ;;4 BIT DELAY TILL READY
        JR      DB13            ;;EXIT
;;
BREAK$X:AND     R7,#"10H        ;;IF BREAK KILL CHARACTER READY FLAG
        CLR     R8              ;;/ AND CLEAR DELAY TIMER
;;
DB13:   OR      R13,R13
        JR      NZ,PROSTAT      ;;JUMP TO PROCESS STATUS BIT
        TM      R14,#1          ;;TEST FOR STATUS START BIT
        JR      NZ,DB13A        ;;JUMP IF NO START BIT
        LD      R13,#1          ;;IF START BIT FOUND THEN COUNT IT
DB13A:  IRET
;;
;;DE-SERIALIZE THE STATUS BYTE
;;
PROSTAT:
        CP      R13,#9          ;;SEE IF WE HAVE A STOP BIT
        JR      LT,PS2
        LD      INTREG+15,R15   ;;TRANSFER TO QUALITY REGISTER
        CLR     R13             ;;CLEAR STATUS BIT COUNTER
        TM      INTREG+15,#DCD  ;;TEST CARRIER STATUS BITS
        JR      NZ,PS1          ;;JUMP IF CARRIER PRESENT
        OR      IMAGE35,#CARRIER ;;/ ELSE SET FOR NO CARRIER
        CLR     INTREG+15       ;;CLEAR SIGNAL QUALITY
        CLR     R3              ;;CLEAR MARK COUNTER
        IRET
PS1:    AND     IMAGE35,#"CARRIER ;;CARRIER PRESENT
        LD      RINGS,INTREG+15   ;;PUT QUALITY INTO S1 REGISTER
        AND     RINGS,#3FH        ;;KEEP ONLY SIGNAL QUALITY BITS
        TM      PFLAGS,#CONEST    ;;CONNECTION ESTABLISHED?
        JR      Z,PS4             ;; // JUMP IF NOT
        TM      INTREG+15,#RDLREQ ;;TEST FOR REMOTE DIGITAL LOOP REQUEST
        JR      NZ,PS3
        OR      PFLAGS3,#ONEZERO  ;;SEND REMOTE DIGITAL LOOPBACK ACKNOWLEDGE
        OR      LPBACK,#S16RDL  ;;SET REMOTE DIGITAL LOOP TEST MODE BIT
        IRET
;;
PS3:    AND     PFLAGS3,#"ONEZERO ;;QUIT SENDING RDL ACK
        TM      INTREG+15,#RDLACK       ;;CHECK FOR RDL ACKNOWLEDGE
        JR      NZ,PS4                  ;;JUMP IF NORMAL
        AND     PFLAGS3,#"SUSM          ;;/ELSE QUIT SENDING UNSCRAMBLED MARKS
PS4:    IRET
;;
PS2:    RR      R14             ;;STATUS BIT INTO CARRY
        RRC     R15             ;;SHIFT CARRY INTO SHIFT REGISTER MSB
        INC     R13             ;;INCREMENT BIT COUNTER
        IRET
;;
;;
;;THE FOLLOWING IS THE CODE WHICH OUTPUTS THE RECEIVED DATA
;;TO THE RS232 PORT AT 1200 BAUD. THE EFFECTIVE CHARACTER RATE IS
;;ABOUT 1215 BAUD  BECAUSE THE STOP BITS ARE SHORTENED TO 
;; 730 MICROSECONDS (NORMAL DATA BITS ARE 833 MICROSECONDS).
;;IT IS ENTERED ONCE FOR EACH 1200 HZ INTERRUPT OF 
;;COUNTER/TIMER (T0).
;;
OUT1219:
        CLR     T0              ;;TIMER 0 SET FOR 1200 BPS CLOCK
        TM      R7,#4           ;;TEST THE MUTE CONTROL BIT
        JR      Z,ODB0          ;;EXIT IF MUTED
        LD      PORT3,R7        ;;OUTPUT DATA BIT TO RS232
ODB0:   EI                      ;;ENABLE INTERRUPTS
        OR      R6,R6           ;;TEST OUTPUT BIT COUNTER FOR ZERO
        JR      NZ,ODB1         ;;JUMP IF IN A CHARACTER (NON-ZERO)
        TM      R7,#10H         ;;SEE IF INPUT CHARACTER IS READY IN HOLDING
        JR      Z,ODB2          ;;JUMP IF NOT AND EXIT
        LD      R9,R10          ;;GET HOLDING REGISTER DATA INTO SHIFT REGISTER
        AND     R7,#"090H       ;;START BIT=0  READY FLAG=0
        LD      R6,#1           ;;LOAD BIT COUNTER=1
ODB2:   IRET                    ;;EXIT
;;
ODB1:   CP      R6,#9           ;;STOP BIT NEXT?
        JR      UGE,DOSTOP      ;;YES, JUMP TO PROCESS IT
        INC     R6              ;;NO, JUST PROCESS NEXT BIT
        RR      R9              ;;SHIFT OVER NEXT BIT TO OUTPUT
        JR      NC,OUTSPC       ;;ZERO OUTPUT IF NO CARRY
OUTMRK: OR      R7,#80H         ;; / ELSE OUTPUT ONE (MARK)
        IRET
;;
DOSTOP: CP      R4,#BRKLEN      ;;IS THIS A BREAK
        JR      UGE,YESBREAK    ;;JUMP IF A BREAK
        CP      R6,#9
        JR      UGT,BKDLY       ;;TRAILING BREAK DELAY IF R6 > 9
        LD      T0,#224         ;;SHORTEN STOP BIT TO 730 MICROSEC.
        CLR     R6              ;;CLEAR BIT COUNTER
        JR      OUTMRK          ;;OUTPUT A MARK
;;
YESBREAK:
        LD      R6,#25          ;;SET TRAILING BREAK DELAY TIMER
        AND     R7,#"10H        ;;CLEAR THE BUFFER READY FLAG
        LD      R10,#0FFH       ;;ALL MARKS TO HOLDING REGISTER
BKDLY:  DEC     R6              ;;COUNT DELAY
        CP      R6,#9           ;;DEBUG AIDS
        JR      NE,OUTSPC       ;;/
        XOR     PORT0,#80H      ;;/
        XOR     PORT0,#80H      ;;/
OUTSPC: TM      LPBACK,#S16DLB+S16RDL ;;CHECK FOR DIGITAL LOOPBACK MODE
        JR      NZ,OUTMRK       ;;SQUELCH RX DATA DURING DIGITAL LOOPBACK
        AND     R7,#"80H        ;;OUTPUT A SPACE
        IRET
;;
;;--------------------------------------------------------------
;;HELP COMMANDS
;;
HELP0:  CALL    WAIT250
        CALL    NEXTNUM         ;;GET DIGIT FOLLOWING THE "?" COMMAND
        CP      R4,#10          ;;CHECK RANGE
        JR      UGT,HLP1        ;;EXIT WITH ERROR IF OUT OF RANGE
        ADD     R4,#3AH         ;;ADD OFFSET FOR RESULT CODE 10=?0
        LD      RESULTS,R4      ;;PUT HELP MSG # IN RESULT CODE REGISTER
HLP1:   JP      GETCOMM         ;;PRINT HELP MESSAGE AS A RESULT CODE.
;;
;;---------------------------------------------------------
;;RESULT CODE TRANSMITTING ROUTINE
;; R0,R1,R2 AND CT1200 ARE USED.
;;
RESULT:
        CP      RESULTS,#'9'            ;;CHECK FOR HELP COMMAND
        JR      UGT,PRNMSG              ;;HELP IS VERBOSE AND NOT QUIET!
        CP      RESULTS,#'5'            ;;CONNECT 1200 RESULT?
        JR      NE,RSL1
        TM      PFLAGS2,#EXR            ;;EXTENDED RESULT MODE?
        JR      NZ,RSL1                 ;;YES, JUMP
        LD      RESULTS,#'1'            ;;NO, CHANGE TO NORMAL CONNECT
RSL1:   TM      OFLAGS,#QUIET           ;;QUIET MODE?
        JR      Z,RSL2                  ;;JUMP IF NOT QUIET
RTN:    RET                             ;;ELSE RETURN WITH NO RESULT CODE
RSL2:   CALL    WAIT250                 ;;WAIT 250 MS & TEST VERBOSE FLAG
        JR      Z,NONVB                 ;;JUMP IF NOT VERBOSE
PRNMSG: CALL    CRLF
        LD      R1,#OKMSG&255
        LD      R2,#'0'-1               ;;START WITH ASCII 0 MINUS 1
        LD      R0,#OKMSG>8             ;;BASE OF MESSAGES
LL005:  LDE     R13,@RR0                ;;GET A MESSAGE CHAR. TO R13
        LD      TXHOLD,R13              ;;INTO TXHOLD
        RL      R13                     ;;SHIFT MSB INTO CARRY
        JR      NC,LL006                ;;JUMP IF NOT DELIMITER
        INC     R2                      ;;ELSE ADVANCE COUNTER
        AND     TXHOLD,#7FH             ;;/ AND ZERO DELIMITER BIT
LL006:  CP      R2,RESULTS              ;;PAST OUR MESSAGE YET?
        JP      UGT,CRLF                ;;YES, FINISH WITH CRLF
        JR      NE,LL007                ;;NOT THERE YET, JUMP
        CP      TXHOLD,#0DH             ;;CHECK FOR END OF LINE CHAR.
        JR      NE,LL006A               ;;JUMP IF NOT END OF LINE
        LD      TXHOLD,CRC              ;;ELSE GET A CARRIAGE RETURN
        CALL    CHAROUT                 ;; / SEND IT
        LD      TXHOLD,LFC              ;; /GET LINE FEED               
LL006A: CALL    CHAROUT                 ;;PRINT IT
LL007:  INCW    RR0                     ;;POINT TO NEXT CHARACTER
        JR      LL005
;;
;;
NONVB:  LD      TXHOLD,RESULTS          ;;GET RAW RESULT CODE
        CALL    CHAROUT                 ;;SEND IT
;;
CRLF:   LD      TXHOLD,CRC              ;;<CR>
        TM      OFLAGS,#VBON            ;;TEST FOR VERBOSE
        JR      Z,CHAROUT               ;;JUST <CR> IF NON-VERBOSE
        CALL    CHAROUT
LFONLY: LD      TXHOLD,LFC              ;;<LF>
;;
;;FALL INTO CHAROUT BELOW....
;;
;; UART TRANSMITTER ROUTINE
;; ENTER WITH DATA TO XMIT IN "TXHOLD"
;;  USES R13,R14,R15,CT1200,BITCOUNT
CHAROUT:
        LD      BITCOUNT,#254
        LD      CT1200,#255
        SCF                     ;;SET CARRY FOR "1" BIT
        CALL    BITOUT          ;;SEND LEADING STOP BIT
        SCF
        CALL    BITOUT          ;;AGAIN FOR A TOTAL OF 2
        CLR     R14             ;;CLEAR PARITY
        TM      USTAT,#EPE      ;;TEST FOR EVEN PARITY
        JR      NZ,LL040        ;;JUMP IF EVEN SELECTED
        INC     R14             ;;ELSE INIT. FOR ODD PARITY
LL040:  LD      R15,#7          ;;ASSUME 7 DATA BITS
        TM      USTAT,#DBITS    ;;CHECK FOR 8 SELECTED
        JR      Z,LL041         ;;JUMP IF 7
        INC     R15             ;;ELSE SET FOR 8
LL041:  LD      R13,USTAT       ;;GET USTAT TO R0
        AND     R13,#DB8        ;;KEEP THE STATE OF DATA BIT 8
        OR      R13,TXHOLD      ;;OR IN THE DATA TO SEND
        RCF                     ;;CLEAR CARRY FOR A ZERO
        CALL    BITOUT          ;; <--- SEND THE START BIT
LL042:  RRC     R13             ;;SHIFT A DATA BIT INTO CARRY FLAG
        JR      NC,LL043        ;;DONT MESS WITH PARITY IF ZERO
        INC     R14
LL043:  CALL    BITOUT          ;; <--- SEND A DATA BIT
        DJNZ    R15,LL042       ;;LOOP TILL ALL DATA BITS SENT
        TM      USTAT,#PE       ;;PARITY ENABLED ??
        JR      Z,LL044         ;;JUMP IF PARITY NOT ENABLED
        RRC     R14             ;;PUT PARITY IN CARRY FLAG
        CALL    BITOUT          ;; <--- SEND PARITY BIT
LL044:  LD      PORT3,#0F0H     ;; NOW SEND THE STOP BIT
        RET
;;
;;                              OBAUD1  OBAUD0
;;                                0       0     INVALID
;; OUTPUTS 1 BIT....              0       1     110 BAUD
;;                                1       0     300 BAUD
;;                                1       1     1200 BAUD
BITOUT: JR      C,LL045
        LD      PORT3,#70H      ;;OUTPUT ZERO BIT
        JR      LL046
LL045:  LD      PORT3,#0F0H     ;;OUTPUT ONE BIT
;;
;;NOW DELAY 1 BIT TIME...
;;
LL046:  TCM     PFLAGS,#OBAUD0+OBAUD1  ;;TEST FOR 1200 BAUD
        JR      NZ,LL047A
        SUB     BITCOUNT,#1             ;;DO 1200 BAUD
        JR      LL047
LL047A: TM      PFLAGS,#OBAUD1  ;;TEST FOR 300 BAUD
        JR      Z,LL047B
        SUB     BITCOUNT,#4             ;;DO 300 BAUD
        JR      LL047
LL047B: SUB     BITCOUNT,#11            ;;DO 110 BAUD
LL047:  CP      CT1200,BITCOUNT ;;COMPARE COUNTER WITH REFERENCE
        JR      NC,LL047        ;;WAIT TILL COUNTER < R3
        RET
;;
;;
;;----------------------------------------------------------
;;
;;THIS IS THE UART RECEIVER
;;REGISTERS USED: R5,R6,R7
;;
IRQ4:                           ;;COME HERE FOR TIMER 0 INTERRUPTS
        CP      IRQ4PTR,#1      ;;DATA BIT COLLECTION IF =1
        JP      EQ,DATABITS
        JP      GT,OUT1219      ;;IF =2 THEN DO 1200 BPS RECEIVER STUFF
OVERFLOW:
        DEC     BITCOUNT        ;;COUNT OVERFLOWS
        JR      NZ,OVRDONE      ;;JUMP IF NOT TOO MANY
        CLR     IRQ3PTR         ;;RESET BACK TO BEGINNING
        AND     IMR,#"10H       ;;KILL T0 INTERRUPTS
        AND     TMR,#"3         ;;AND STOP T0
OVRDONE: IRET
;;
;;
FINDBAUD:
        AND     TMR,#0FCH       ;;STOP T0
        LD      T0,#255         ;;LOAD T0
        LD      PRE0,#0EDH      ;;DIV. BY 59... 64.02 US PER COUNT
        LD      IRQ3PTR,#1      ;;CHANGE IRQ3 POINTER TO "COMPUTEBAUD"
        CLR     IRQ4PTR         ;;POINT T0 INTERRUPT TO OVERFLOW HANDLER
        LD      BITCOUNT,#2     ;;USED TO COUNT OVERFLOWS OF T0
        DI
        AND     IRQ,#"18H       ;;KILL PENDING T0 AND RS232 DATA INTERRUPT
        OR      TMR,#3          ;;START T0 RUNNING
        OR      IMR,#18H        ;;ENABLE T0 AND RS232 DATA INTERRUPTS
        IRET
;;
;;
COMPUTEBAUD:                    ;;COME HERE ON NEXT 1->0 TRANSISTION IN "A"
        AND     TMR,#"03        ;;STOP T0
        LD      IRQ3PTR,#2      ;;IRQ3 NOW POINTS TO "STARTBIT"
        LD      R14,#80H        ;;ASSUME LSB WAS 1 FOR THE "A"
        OR      PFLAGS,#IBAUD0+IBAUD1  ;;SET THE BAUD SELECT BITS  (11)
        CP      BITCOUNT,#1     ;;TEST FOR OVERFLOWS (110 BAUD OR LESS)
        LD      BITCOUNT,#2     ;;PRESET BIT COUNTER TO 2 ONLY FOR THE "A"
        JR      GT,CBD0         ;;JUMP IF GREATER THAN 110 BAUD
        XOR     PFLAGS,#IBAUD1  ;;SET BAUD BITS FOR 110  (01)
        JR      CBD1            ;;THEN EXIT
CBD0:
        CP      T0,#213         ;;COMPARE WITH 750 BAUD VALUE
        JR      UGT,CBD1        ;;JUMP IF 1200 BAUD
        XOR     PFLAGS,#IBAUD0  ;;NOW ASSUME 300 BAUD       (10)
        JR      CBD1
;;
;;
;;
IRQ3:   AND     IMR,#"8         ;;KILL RS232 DATA INTERRUPTS
        EI                      
        CP      IRQ3PTR,#1      ;;1= FIRST 1 TO 0 TRANSITION IN THE "A"
        JR      EQ,COMPUTEBAUD
        JR      LT,FINDBAUD     ;;0= "A" START BIT INTERRUPT
;;
STARTBIT:                       ;;COME HERE WHEN IRQ3PTR=2
        CLR     BITCOUNT
CBD1:   AND     TMR,#"3         ;;STOP T0
        LD      PRE0,#(32*4)+1  ;;SET FOR DIVIDE BY N= 32 CONT.
        LD      T0,#12          ;;SET FOR 1/2 BIT TIME AT 1200 BAUD
        TCM     PFLAGS,#(IBAUD0+IBAUD1)         ;;TEST FOR 1200 BAUD
        JR      Z,CBD2          ;;JUMP IF 1200
        LD      T0,#48          ;;SET FOR 1/2 BIT TIME AT 300 BAUD
        TM      PFLAGS,#IBAUD0  ;;CHECK FOR 300
        JR      Z,CBD2          ;;JUMP IF 300
        LD      T0,#131         ;;ELSE SET FOR 110
CBD2:   OR      TMR,#3          ;;START THE BIT TIMER
        LD      IRQ4PTR,#1      ;;SET IRQ4 (T0) TO DATA BIT ROUTINE
        OR      LPBACK,#80H     ;;SET 1/2 BIT TIME FLAG
        DI
        AND     IRQ,#"10H       ;;KILL PENDING T0 INTERRUPT
        LD      IMR,#30H        ;;ENABLE T0 INTERRUPT (IRQ4), KILL STARTBIT INT
CBD3:   IRET
;;
DATABITS:                       ;;COME HERE EVERY TIME T0 TIMES OUT
        TM      USTAT,#IBF      ;;BUFFER FULL?
        JP      NZ,LL039        ;; / QUIT PROCESSING IF BUFFER IS FULL
        XOR     LPBACK,#80H     ;;FLIP THE 1/2 BIT TIME FLAG
        JR      MI,LL027        ;;PROCESS ONLY IF IN BIT CENTER
        LD      R15,PORT3       ;;GET RS232 DATA BIT INTO R15, BIT 0
        EI                      ;;ALLOW NESTED INTRRUPTS
        CP      BITCOUNT,#9     ;;IN PARITY OR STOP?
        JR      UGE,LL031       ;;YES, JUMP
        CP      BITCOUNT,#0     ;;SEE IF WE ARE STILL IN THE START BIT
        JR      EQ,LL028        ;;IF SO THEN JUMP
        RRC     R15             ;;PUT IT DATA BIT IN CARRY FLAG
        RRC     R14             ;;SHIFT IT INTO RX SHIFT REGISTER
LL026:  INC     BITCOUNT
LL027:  IRET
;;
LL028:  TM      R15,#1          ;;TEST FOR SPACE INPUT
        JR      Z,LL026         ;;JUMP IF VALID START BIT
LL029:  DI
        LD      IMR,#28H        ;;KILL T0 INT, ENABLE STARTBIT
        AND     IRQ,#"18H       ;;KILL PENDING IRQ3 & 4
LLDONE: IRET
;;
LL031:  DI
        AND     IRQ,#"8         ;;KILL PENDING STARTBIT INTERRUPT
        LD      IMR,#28H        ;;DISABLE T0, ENABLE STARTBIT INTERRUPTS
        TM      PORT3,#DTR      ;;TEST DTR LEAD
        EI
        JR      Z,LL033         ;;JUMP IF DTR FALSE TO IGNORE COMMANDS
        TM      OFLAGS,#STUPID  ;;SEE IF WE TAKE COMMANDS
        JR      NZ,LL033        ;;JUMP IF STUPID BIT IS SET
        RRC     R15             ;;PUT RS232 DATA BIT INTO CARRY
        RLC     R5              ;;..THEN INTO LSB OF R5 (BITS 8&9 GO HERE)
        LD      RXHOLD,R14      ;;THIS IS THE COMPLETE RECEIVED CHAR.
        OR      USTAT,#RRF
        INC     BITCOUNT
        LD      R6,#'A'
        INC     CHARS
        JR      MI,LL032        ;;JUMP IF 1ST CHAR "A"
        JR      NZ,LL034        ;;JUMP IF PAST THE "T"
PARITY:
        LD      R6,PFLAGS               ;;GET BAUD RATE FLAGS
        SWAP    R6                      ;;MOVE THEM OVER TO OUTPUT SIDE
        AND     R6,#(OBAUD0+OBAUD1)     ;;KEEP ONLY OUTPUT BAUD CONTROL BITS
        AND     PFLAGS,#"(OBAUD0+OBAUD1);;CLEAR OLD OUTPUT BAUD BITS
        OR      PFLAGS,R6               ;;TRANSFER INPUT BAUD TO OUTPUT BAUD
;;                              ;;NOW PROCESS THE "T" OR "/"
        LD      R6,CHAR1        ;;GET THE "A"
        ADD     R6,R6           ;;SHIFT MSB INTO CARRY
        RLC     R5              ;;THEN INTO R5, BIT 0
        LD      R6,RXHOLD       ;;GET THE "T"
        ADD     R6,R6           ;;SHIFT MSB INTO CARRY
        RLC     R5              ;;THEN INTO R5, BIT 0
        CP      R6,#'/' < 1     ;;SEE IF ITS A "/"
        JR      EQ,REPEATIT     ;;DONT DO PARITY PROCESSING IF REPEAT
        AND     R5,#0FH         ;;KEEP LOW 4 BITS
        LD      R7,#FORMAT&255  ;;MAKE POINTER TO FORMAT TABLE
        LD      R6,#FORMAT>8
        ADD     R7,R5           ;;INDEX INTO TABLE
        LDE     R5,@RR6         ;;GET CHAR. FORMAT CODE
        AND     USTAT,#"(PE+EPE+DBITS+DB8)
        OR      USTAT,R5        ;;PUT CODE INTO "USTAT"
        LD      R6,#'T'
        TM      RXHOLD,#20H             ;;SEE IT THIS WAS LOWER CASE
        JR      Z,LL032                 ;;JUMP IF UPPER CASE
        XOR     USTAT,#EPE              ;;FLIP PARITY FLAG IF LOWER CASE
LL032:  LD      CHAR1,RXHOLD    ;;SAVE THIS CHAR
        AND     RXHOLD,#5FH     ;;STRIP PARITY AND LOWER CASE BIT
        CP      RXHOLD,R6       ;;IS CHAR CORRECT? ( A OR T)
        JR      EQ,LLDONE2      ;;YES, DONE WITH THIS ONE
LL033:  DI
        LD      CHARS,#-2       ;;FAULTY "A" ,"T" OR "/"
        CLR     IRQ3PTR         ;; / ABORT BACK TO "FINDBAUD" AGAIN
        JR      LL029           ;;/  TO LOOK FOR AN "A"
;;
LL034:  AND     RXHOLD,#7FH     ;;STRIP PARITY
        CP      RXHOLD,CRC      ;;CARRIAGE RETURN?
        JR      EQ,LL038        ;;
        CP      CHARS,#MAXCHARS ;;BUFFER FULL?
        JR      ULE,LL036       ;;JUMP IF NOT FULL
        OR      USTAT,#OVL      ;;ELSE SET OVERFLOW FLAG
LL035:  DEC     CHARS           ;;DISCARD THE CHARACTER
        IRET                    ;;DONE....
;;
LL036:  CP      RXHOLD,#' '     ;;SPACE OR CONTROL CHAR?
        JR      ULE,LL035       ;;DUMP SPACE OR CONTROL CHAR
        CP      RXHOLD,#7FH     ;;RUBOUT?
        JR      EQ,LL035        ;;DUMP IT TOO
        CP      RXHOLD,#60H     ;;LOWER CASE?
        JR      ULT,LL037
        SUB     RXHOLD,#20H     ;;CONVERT TO UPPER CASE
LL037:  LD      R6,CHARS        ;;GET CHARACTER COUNT
        ADD     R6,#BUFFER-1    ;;INDEX INTO BUFFER
        LD      @R6,RXHOLD      ;;PUT CHARACTER IN BUFFER
        IRET                    ;;FINISHED PROCESSING 1 CHARACTER
;;
LL038:  DEC     CHARS                   ;;ADJUST CHARACTER COUNT
LL039:  OR      USTAT,#IBF              ;;SET INPUT BUFFER FULL FLAG
        DI                              ;;DISABLE INTERRUPTS
        LD      IMR,#20H                ;;KILL ALL UART INTERRUPTS
LLDONE2:IRET                            ;;**FINISHED PROCESSING COMMAND LINE 
;;
REPEATIT:
        LD      CHARS,LCHARS            ;; <--- "A/" COMMAND EXECUTED HERE
        JR      LL039
;;      
;; **  END OF UART RECEIVER ROUTINE
;;
;;-------------------------------------------------------------
;;
NO$CHAR:                ;;COME HERE WHEN NO TX CHAR AVALABLE
        CLR     COUNTER
        TM      IRQ,#8          ;;CHECK FOR START BIT
        JR      Z,SINEWAVE      ;;EXIT IF NO START BIT
        TM      PFLAGS,#ONLINE  ;;ARE WE ONLINE?
        JR      Z,SINEWAVE      ;;/IF NOT THEN DONT DO CHARACTER STUFF
        LD      COUNTER,#76     ;;LOAD THE COUNTER
        JR      SINEWAVE
;;
TEST$STOP:              ;;COME HERE DURING STOP BIT
        LD      R11,R4          ;;TRANSFER TO DATA TO HOLDING REGISTER, R11
        OR      PFLAGS2,#TCA    ;;SET TX CHARACTER AVALABLE FLAG
        LD      TXDATA,PORT3    ;;SAMPLE THE STOP BIT
        AND     IRQ,#"8         ;;CLEAR START BIT FLAG
        TM      TXDATA,#1       ;;CHECK FOR STOP BIT PRESENT
        JR      NZ,SINEWAVE     ;;EXIT IF A STOP BIT IS PRESENT
        LD      TXDATA,#242     ;;15 BITS TIMES 2 +2
;;
IN$BREAK:                       ;;COME HERE DURING A BREAK
        LD      COUNTER,#80H    ;;CONTINUE TO SET BREAK FLAG
        CP      TXDATA,#0       ;;BREAK EXTENDER TIMED OUT?
        JR      Z,IB2           ;;JUMP IF SO
        SUB     TXDATA,#2       ;;ELSE COUNT DOWN BY 2 (LEAVE LSB ALONE)
        JR      SINEWAVE        ;;THEN EXIT
IB2:    TM      PORT3,#1        ;;STILL DOING A BREAK?
        JR      Z,SINEWAVE      ;;EXIT IF STILL DOING BREAK
        COM     TXDATA          ;;FORCE MARK ON TXDATA
        CLR     COUNTER         ;;CLEAR THE COUNTER
        AND     IRQ,#"8         ;;RESET START BIT FLAG
        JR      SINEWAVE        ;;EXIT
;;
;;
;;
TIMEBASE:
        SRP     INTREG
;;
;;ASYNC TO SYNC CONVERTER
;;
        DEC     COUNTER         ;;DEC. SAMPLE COUNTER
        JR      MI,NO$CHAR      ;;NOTHING DOING IF MINUS
        JR      Z,TEST$STOP     ;;IN STOP BIT IF ZERO
        JR      OV,IN$BREAK     ;;DOING A BREAK IF OVERFLOW
        TM      COUNTER,#7      ;;ASYNC SAMPLE IF LOW 3 BITS ZERO
        JR      NZ,SINEWAVE
        LD      TXDATA,PORT3    ;;SAMPLE TX DATA NOW
        LD      R0,TXDATA
        RRC     R0              ;;SHIFT INTO TX DATA BIT INTO CARRY
        RRC     R4              ;;THEN INTO BIT 7 OF R4
;;
SINEWAVE:
        TM      PFLAGS3,#TXON   ;;SEE IF TRANSMITTER IS ON
        JR      NZ,A2S
        JP      @RR2            ;;DO NOTHING IF TX IS OFF
A2S:    LD      R0,R13          ;;POINT TO SINE TABLE HIGH BYTE R0
        LD      R1,R12          ;;GET SINE POINTER TO R1
        LDE     R2,@RR0         ;;GET THE SINE R2
        LD      DAC,R2          ;;OUT TO THE DAC
        ADD     R12,R10         ;;ADD FREQUENCY VARIABLE
        CLR     R2              ;;CLEAR HIGH BYTE OF STATE POINTER
        JP      @RR2
;;
;;END OF THE 1200 BAUD TIMEBASE ROUTINE
;;
;;------------------------------------------------------------------
;;
;;***** THIS IS THE LOW SPEED FSK TIMEBASE AND MODEM TRANSMITTER ROUTINE **
;;***** ALSO, THE COMPLETE FSK RECEIVE LOGIC IS HERE.
;;
LOW$TIMEBASE:
        SRP     INTREG
        TM      PFLAGS3,#TXON   ;;TRANSMITTER ON ?
        JR      Z,LTB           ;;BYPASS CARRIER GENERATOR IF OFF
        LD      R0,#SINE256 > 8
        LD      R1,R12
        LDE     R0,@RR0         ;;GET SINE VALUE FROM TABLE
        LD      DAC,R0          ;;OUT TO REAL WORLD
;;
LTB:    LD      R0,PORT3        ;;GET THE TRANSMIT DATA FROM RS232 LINE
        RR      R0              ;;PUT THE BIT INTO CARRY AND BIT 7
        TM      LPBACK,#S16DLB  ;;TEST FOR DIGITAL LOOP BACK MODE
        JR      Z,LTB2          ;;JUMP IF NORMAL MODE
        TM      IMAGE35,#CARRIER
        JR      NZ,FSKMARK
        TM      PORT2,#RDATA    ;;LOOP RX DATA BACK TO XMITTER FOR DLB
        JR      Z,FSKSPACE
        JR      FSKMARK
LTB2:   TM      PFLAGS,#ONLINE  ;;SET ZERO FLAG IF NOT ONLINE
        JR      ULE,FSKMARK     ;;JUMP IF CARRY OR ZERO FLAGS SET (SUPER TRICK)
;;
FSKSPACE:ADD    R13,R7          ;;DO SPACE FREQUENCY
        ADC     R12,R6
        JR      FSKDONE
FSKMARK:TCM     PFLAGS3,#SBREAK ;;SEND BREAK?
        JR      Z,FSKSPACE      ;;  / YES, SEND SPACE INSTEAD OF MARK
        ADD     R13,R9          ;;ELSE DO MARK FREQUENCY
        ADC     R12,R8
;;
FSKDONE:
        DJNZ    R5,LTB0         ;;DIVIDE 4800 HZ DOWN TO 1200 HZ
        LD      R5,#6
        CP      CT1200,#0
        JR      Z,LTB0
        DEC     CT1200          ;;MAINTAIN 1200 HZ COUNTER
LTB0:   DJNZ    R14,NO$SAMPLE   ;;THIS IS THE BAUD COUNTER (R14)
        LD      R14,R15         ;;RELOAD BAUD COUNTER WITH 4 OR 11
LTB1:   DEC     COUNTER         ;;DECREMENTS AT 8 TIMES BIT RATE
        JR      MI,NO$CHR       ;;JUMP IF NO CHARACTER AVALABLE
        JR      Z,IN$STOP       ;;JUMP IF IN STOP BIT
        TM      COUNTER,#7      ;;SEE IF LOW 3 BITS ARE ZERO
        JR      NZ,NO$SAMPLE    ;;DONT SAMPLE BIT IF BITS NOT ALL ZERO
        RL      R0              ;;PUT BIT INTO CARRY
        RRC     R4              ;;THEN INTO SERIAL TO PARALLEL REGISTER
        SRP     PRIMREG
        IRET
;;
NO$SAMPLE:
        TM      PFLAGS,#DPLX    ;;TEST FOR HALF DUPLEX
        JR      Z,HALFDPX       ;;JUMP IF HALF DUPLEX
        LD      R0,#255         ;;IF FULL MAKE RECV THINK TX BIT IS MARK
HALFDPX: SRP    PRIMREG         ;;BACK TO PRIMARY REGISTERS NOW
        TM      PFLAGS,#ONLINE
        JR      Z,NO$SAM1       ;;IF OFF-LINE DONT MESS WITH R7
        OR      R7,#80H         ;;ASSUME MARK
        TM      R7,#4           ;;TEST FOR MUTED RECEIVER
        JR      Z,NO$SAM2       ;;PUT OUT MARK IF MUTED
        TM      LPBACK,#S16DLB  ;;DIGITAL LOOP BACK?
        JR      NZ,NO$SAM2      ;; / FORCE MARK IF DLB MODE
        TM      PORT2,#RDATA    ;;TEST RECEIVED DATA BIT
        JR      NZ,NO$SAM2
        AND     R7,#"80H        ;;ELSE RECEIVE A SPACE
NO$SAM2:OR      INTREG+0,#7FH   ;;FORCE UN-NEEDED BITS TO ALL ONES 
        AND     R7,INTREG+0     ;;MIX IN THE TRANSMITTED DATA (BIT 7) FOR ECHO
        LD      PORT3,R7        ;;OUTPUT TO RS232 IF NOT MUTED
NO$SAM1:IRET
;;
NO$CHR:
        OR      IMAGE35,#CARRIER ;;ASSUME NO CARRIER
        TM      PORT3,#STATUS   ;;TEST FOR CARRIER DETECT
        JR      Z,NO$CHR1       ;;JUMP IF NO CARIER
        AND     IMAGE35,#"CARRIER  ;;ELSE SET CARRIER DETECT FLAG
;;
NO$CHR1:CLR     COUNTER
        TM      IRQ,#8          ;;CHECK FOR START BIT
        JR      Z,NO$SAMPLE
        LD      COUNTER,#76     ;;LOAD SAMPLING CONTROL COUNTER
        SRP     PRIMREG
        IRET
;;
IN$STOP:
        LD      R11,R4          ;;PUT CHAR IN HOLDING REGISTER
        OR      PFLAGS2,#TCA    ;;SET TRANSMIT CHARACTER AVALABLE FLAG
        AND     IRQ,#"8         ;;KILL PENDING STARTBIT FLAG
        SRP     PRIMREG
        IRET    
;;
;;
;;
;;
;;----------------------------------------------------------------
;;THESE HARDWARE UPDATE OR REFRESH ROUTINES USE R0,R1,R2,TXHOLD.
;;NOTE: "TXHOLD" IS USED ONLY AS A GP REGISTER HERE.
;;
UD3435: CALL    UPDATE35        ;;UPDATE BOTH 34 AND 35
UPDATE34:                       ;;WRITE IMAGE34 TO HARDWAARE
        LD      R2,#30H         ;;CON2 LOW
        LD      TXHOLD,#20H     ;;SET FOR P35 LATCH
        CALL    STROB           ;;STROBE IT INTO LATCH  
UD34XX: LD      TXHOLD,#40H     ;;SET FOR P36 LATCH (USED TO BE P34)
        LD      R0,IMAGE34
        RL      R0
        RL      R0
        LD      R1,#50H         ;;DONT DO VOL0,VOL1 BITS
        CALL    UPDATEB         ;;SEND IMAGE34 OUT TO LOW 6 LATCH BITS
        LD      TXHOLD,#20H     ;;SET FOR P35 LATCH
        LD      R2,#30H+80H     ;;CON2 HIGH
        CALL    STROB           ;;SET CON2 HIGH AFTER ALL P34 BITS CHANGED
        CLR     R2
D830US: DJNZ    R2,D830US       ;;830 MICROSECOND DELAY
        RET
;;
;;
;;--------------------------------------------------------
;; THESE ROUTINES SET OR CLEAR THE AA-LED AND RING INDICATOR RS232 PIN
;; 4 BYTES OF STACK ARE USED
;; R0,R1,R2,PORT0,PORT3 ARE DISTURBED
;;
CLRRI:  OR      IMAGE35,#(AA$LED+RING)  ;;RI AND AA-LED OFF
        JR      SETRI0  
SETRI:  AND     IMAGE35,#"(AA$LED+RING) ;;RI AND AA-LED ON
SETRI0: CP      RINGN,#0                ;;SEE IF ENABLED TO ANSWER
        JR      EQ,UPDATE35             ;; / IF NOT, DO HARWARE UPDATE NOW
        XOR     IMAGE35,#AA$LED         ;; / ELSE INVERT AA-LED STATE
;;
;;FALL INTO "UPDATE35" BELOW.
;;---------------------------------------------------------
;;
UPDATE35:                       ;;WRITE IMAGE35 TO HARDWARE
        LD      R0,IMAGE35
        LD      TXHOLD,#20H
        LD      R1,#70H         ;;FIRST ADDRESS
UPDATEB:CP      TXHOLD,#40H     ;;TEST FOR P36 LATCH ADDRESS
        JR      EQ,UPDATEE      ;;/ NO RESTRICTIONS ON P36 LATCH
        CP      R1,#60H         ;;CARRIER DETECT ADDRESS? (FORBIDDEN)
        JR      EQ,UPDATED      ;;DONT UPDATE CARRIER DETECT STATUS!
        CP      R1,#30H         ;;CON2 ADDRESS? (FORBIDDEN)
        JR      EQ,UPDATED      ;;DONT UPDATE CON2 BIT
UPDATEE:LD      R2,R0           ;;GET BIT TO OUTPUT FROM R0
        AND     R2,#80H
        OR      R2,R1           ;;COMBINE WITH ADDRESS
        CALL    STROB           ;;SEND TO TO ADDRESSABLE LATCH
        LD      R2,#42          ;;WAIT 500 CPU CYCLES TO KEEP FROM DISABLING
UPDATEC:DJNZ    R2,UPDATEC      ;; / ... INTERRUPTS TOO OFTEN
UPDATED:RL      R0              ;;GET NEXT BIT
        SUB     R1,#10H         ;;NEXT ADDRESS
        JR      PL,UPDATEB
        RET
;;
;;ADDRESS AND DATA IN R2, STROBE ADDRESS IN TXHOLD
;;
STROB:  PUSH    IMR             ;;SAVE INTERRUPT MASK
        DI
        LD      PORT0,R2        ;;OUTPUT ADDRESS AND DATA TO THE LATCH
        COM     TXHOLD          ;;
        AND     PORT3,TXHOLD    ;;BRING STROBE LOW
        COM     TXHOLD  
        OR      PORT3,TXHOLD    ;;THEN HIGH AGAIN
        POP     IMR             ;;RESTORE IMR
        RET
;;-----------------------------------------------
;;SPEAKER ON/OFF AND VOLUME CONTROL ROUTINES
;;
SPKCRL: AND     IMAGE34,#"(VOL0+VOL1)   ;;SET VOLUME TO ZERO
        TM      PFLAGS3,#(SPON+SPON2)   ;;SEE IF USER WANTS SPEAKER ON
        JR      Z,SETVOL                ;;JUMP IF NEVER ON
        JR      MI,ALWAYS               ;;JUMP IF ALWAYS ON
        TM      PFLAGS2,#CDLED          ;;CHECK FOR CARRIER
        JR      NZ,SETVOL               ;;SPEAKER OFF IF CARRIER PRESENT
ALWAYS: LD      R4,PFLAGS2              ;;/ ELSE GET USER VOLUME LEVEL
        AND     R4,#(VOL0+VOL1)
        OR      IMAGE34,R4              ;;PUT VOLUME INTO IMAGE34
SETVOL: LD      TXHOLD,#40H     ;;ADDRESS P36 LATCH
        LD      R2,IMAGE34      ;;GET VOLUME BITS TO R2
        AND     R2,#80H         ;;KEEP MSB (VOL1)
        OR      R2,#70H
        CALL    STROB           ;;UPDATE SPEAKER VOLUME HARDWARE
        LD      R2,IMAGE34
        RL      R2
        AND     R2,#80H         ;;KEEP VOL0
        OR      R2,#60H         ;;ADDRESS VOL0
        JR      STROB           ;;UPDATE VOL0
;;
;;
;;
;;------------------------------------------------------------------
;;  **CAUTION, DO NOT CROSS NEXT PAGE BOUNDRY**
;;NEXT ADDRESS MUST BE 0F00H
;;----------------------------------------------------------------------
;;
        ORG     0F00H           ;;<---------- NEW PAGE ------<<<        
;;
XTABLE: DB      0               ;;X0
        DB      EXR             ;;X1  EXTENDED RESULTS (CONNECT 1200)
        DB      WDT+EXR         ;;X2  WAIT FOR DIAL TONE PLUS X1 FEATURES
        DB      BYCXR+EXR       ;;X3  WAIT FOR CARRIER OR BUSY PLUS X1 FEATURES
        DB      WDT+BYCXR+EXR   ;;X4  ALL OF THE ABOVE FEATURES
;;
;;**** WARNING **** NEXT ADDRESS MUST BE 0F0CH *************
;;
        ORG     0F0CH
;;
ZZZZZZ: JP      START           ;;CPU THINKS THIS IS 000C AFTER RESET!!
START:  LD      SPH,#OBAUD0+OBAUD1  ;;COME HERE AFTER COLD START <-----
        CLR     SPL
        LD      PORT0,#0FFH     ;;PRESET PORT0 TO AVOID GITCHING ROM ADDRESS
        LD      P01M,#16H       ;;PORT 1 = ADR/DATA / PORT 0 = OUTPUT/ADR
        LD      PORT3,#0F0H     ;;PRESET PORT3 TO AVOID GLITCHING RS232 DATA
        LD      P3M,#9          ;;SIMPLE I/O MODE FOR PORT 3 (P34=-DM)
        LD      P2M,#3          ;;PORT 2 BITS 0,1 INPUTS, 2-7 OUTPUTS
SLAVETEST:
        TM      PORT3,#CLOCK    ;;WAIT TILL SLAVE PROCESSOR IS READY
        JR      NZ,SLAVETEST    ;;// TAKES 0.5 SECONDS FOR SLAVE TO DO SELFTEST
WARM:   LD      7FH,#4          ;; POINTER FOR TEST AND CLEAR OPERATION
LL000:  INC     7FH
        CLR     @7FH            ;;CLEAR REGISTER POINTED TO BY 7FH
        CLR     4               ;;CLEAR 4
RAMTEST:CP      4,@7FH          ;;COMPARE REFERENCE BYTE WITH TEST BYTE
RAMFAIL:JR      NE,RAMFAIL      ;;HALT HERE IF BAD RAM LOCATION
        INC     @7FH            ;;INCREMENT LOCATION UNDER TEST
        INC     4               ;;INCREMENT REFERENCE BYTE
        JR      NZ,RAMTEST      ;;LOOP 256 TIMES
        CP      7FH,#7EH        ;;DONE ALL LOCATIONS?
        JR      NE,LL000        ;;  /JUMP IF NOT
;;
        SRP     PRIMREG         ;;INITIALIZE THE "S" REGISTERS
        LD      R14,#INITDATA > 8
        LD      R15,#INITDATA & 255
        LD      R12,#SR         ;;POINT R12 TO S REGISTERS
        LD      R13,#17         ;;NUMBER OF REGISTERS TO INIT.
LL001A: LDEI    @R12,@RR14      ;;DO INIT.
        DJNZ    R13,LL001A
        AND     SPH,#OBAUD0+OBAUD1+IBAUD0+IBAUD1  ;;KEEP LAST USER BAUD RATE
        OR      PFLAGS,SPH      ;;RECOVER USER BAUD RATE IF Z COMMAND
        AND     SPL,#PE+EPE+DBITS+DB8  ;;KEEP PARITY AND WORD LENGTH INFO
        LD      USTAT,SPL       ;;RECOVER USERS WORD LEN. AND PARITY
        LD      SPL,#STACK      ;;SET STACK POINTER
;;
;;INITIALIZE COUNTERS-TIMERS
;;
        LD      IPR,#0BH
        LD      IMR,#20H                ;;ENABLE IRQ5 
        LD      PRE1,#7                 ;;MOD N, INTERNAL, DIVIDE BY 1
        LD      T1,#MC9600              ;;MASTER 9600 HZ CLOCK (IRQ5)
        LD      TMR,#0CH                ;;START T1
        LD      IRQ5VH,#TIMEBASE > 8
        LD      IRQ5VL,#TIMEBASE & 255  ;;SET T1 INTERRUPT DESTINATION
        CALL    ZERO$FREQ               ;;KILL TRANSMITTER
        LD      TXFLAG,#TXON            ;; "C1" COMMAND DEFAULT
;;
;;NOW READ IN THE OPTION SWITCHES
;;
        LD      R15,#SW9        ;;ADDRESS OF SWITCH 9 (BELL/CCITT)
LL002:  LD      PORT0,R15       ;;SEND OUT THE ADDRESS TO THE MUX
        SCF                     ;;SET CARRY
        TM      PORT2,#MUXOUT   ;;BIT 1 OF PORT 2 IS THE DATA BIT WE WANT
        JR      NZ,LL002A       ;;JUMP IF BIT IS SET
        RCF                     ;;ELSE CLEAR CARRY BIT
LL002A: RLC     OFLAGS          ;;SHIFT THE BIT (CARRY) INTO LSB OF OFLAGS
        SUB     R15,#10H        ;;NEXT ADDRESS
        JR      NZ,LL002        ;;LOOP 7 TIMES FOR ALL SOFT OPTION SWITCHES
;;
        LD      PFLAGS3,#SPON   ;;DEFAULT TO SPEAKER ON TIL CARRIER
        LD      PFLAGS2,#VOL1   ;;DEFAULT TO L2 LOUDNESS
        TM      OFLAGS,#AAON    ;;TEST FOR AUTO-ANSWER
        JR      NZ,LL002B       ;;JUMP IF SET TO AUTO-ANSWER
        CLR     RINGN           ;;ELSE CLEAR THE S0 REGISTER
LL002B: JP      MOREINIT        ;;COMPLETE INITIALIZATION ELSEWHERE
;;
RESTART:DI                      ;;COME HERE AFTER Z COMMAND
        LD      SPH,PFLAGS      ;;SAVE USERS BAUD RATE
        LD      SPL,USTAT       ;;SAVE USERS PARITY AND WORD LENGTH
        JP      WARM
;;------------------------------------------------------
;;THIS IS THE COMMAND VECTOR TABLE
;;
CMDTBL:
        DB      'A'             ;;ANSWER PHONE WITHOUT RINGS
        DW      A$COMM
        DB      'B'             ;;B1=BELL   B0=CCITT
        DW      BELL$CCITT
        DB      'C'             ;;CARRIER ON/OFF 
        DW      SETCARRIER
        DB      'D'             ;;DIAL
        DW      DIAL
        DB      'E'             ;;LOCAL ECHO ON/OFF
        DW      SETECHO
        DB      'F'             ;;FULL DUPLEX
        DW      SETFULL
        DB      'H'             ;;SWITCH HOOK CONTROL
        DW      HOOKCRL
        DB      'I'             ;;PRODUCT ID COMMAND
        DW      PRODUCT$ID
        DB      'L'             ;;SPEAKER LOUDNESS CONTROL
        DW      LOUDNESS
        DB      'M'             ;;MONITOR SPEAKER CONTROL
        DW      SPCRL
        DB      'S'             ;;DIRECT SET COMMAND
        DW      SETADR
        DB      'V'             ;;VERBOSE MODE
        DW      SETVERBOSE
        DB      'O'             ;;GO ONLINE
        DW      GO$ONLINE
        DB      'Y'             ;;LONG SPACE DISCONNECT OPTION
        DW      LS$DISC
        DB      'Z'             ;;SOFT RESET
        DW      RESTART
        DB      'Q'             ;;QUIET MODE (NO RESULT CODES)
        DW      SETQUIET
        DB      'X'             ;;EXTENDED RESULT CODE ENABLE
        DW      EXRESULT
        DB      '='             ;;DATA PART OF SET COMMAND
        DW      SETRAM
        DB      '?'             ;;GET CONTENTS OF ADDRESS
        DW      GETRAM
        DB      ':'             ;;GLOBAL SET COMMAND
        DW      ANYLOC
CMDT2:  
        DB      '/'             ;;WAIT 125 MS
        DW      W120
        DB      ','             ;;PAUSE
        DW      WAIT2SEC
        DB      'T'             ;;TOUCH TONE ENABLE
        DW      ENABTONE
        DB      'P'             ;;PULSE DIAL ENABLE
        DW      ENABPULSE
        DB      'R'             ;;REVERSE ORIG./ANSW. MODES
        DW      REVERSE
        DB      ';'             ;;TERMINATE DIAL COMMAND
        DW      STOP$DIAL
        DB      '!'             ;;FLASH
        DW      FLASH
        DB      '@'             ;;WAIT FOR NON-MODEM ANSWER
        DW      NMANSW
        DB      'W'             ;;WAIT FOR SECOND DIAL TONE
        DW      WAITDIAL
        DB      0               ;;END OF TABLE MARKER.
;;
        END                     ;;END OF PROGRAM CODE
